from _create_subdirectories import _create_subdirectories
from _get_documenting_directories import _get_documenting_directories
from populate_sphinx_directory import populate_sphinx_directory
import os


def build_doc_tree(ABJADPATH, api_doc_path, interactive):
   '''Walk entire Abjad codebase and build corresponding Sphinx doc tree.
   '''

   # walk every toplevel directory that should document
   print 'Building TOC tree ...'
   for documenting_directory in _get_documenting_directories():
      documenting_directory = os.path.join(ABJADPATH, documenting_directory)
      for current_root, directories, files in os.walk(documenting_directory):
         
         # remove directories that should not document
         for directory in directories[:]:
            if directory in ('.svn', 'test'):
               directories.remove(directory)
            elif directory.startswith('_'):
               directories.remove(directory)
            elif 'ParentageInterface' in directory:
               directories.remove(directory)
         directories.sort()
      
         # remove files that should not document
         for file in files[:]:
            if file.startswith('_'):
               files.remove(file)
            elif not file.endswith('.py'):
               files.remove(file)
         files.sort()

         # make subdirectories in docs if needed
         abjad_subdir = current_root.split('abjad')[-1].strip(os.path.sep)
         root_dir = os.path.join(api_doc_path, abjad_subdir)
         _create_subdirectories(root_dir, directories, interactive)

         # populate sphinx directory with sphinx files
         populate_sphinx_directory(root_dir, abjad_subdir, files, interactive)



from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad import Fraction


def round_interval_bounds_to_nearest_multiple_of_rational(intervals, rational):
    assert all_are_intervals_or_trees_or_empty(intervals)
    assert isinstance(rational, (int, Fraction)) and 0 < rational

    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree:
        return tree

    intervals = []
    for interval in tree:
        low = Fraction(int(round(interval.low / rational))) * rational
        high = Fraction(int(round(interval.high / rational))) * rational
        if low == high:
            high = low + rational
        intervals.append(interval.shift_to_rational(low).scale_to_rational(high - low))

    return IntervalTree(intervals)



from abjad.tools.importtools._import_structured_package import _import_structured_package

_import_structured_package(__path__[0], globals( ))

from _FlexEqualityComparator import _FlexEqualityComparator
from _Immutable import _Immutable
from _ImmutableDictionary import _ImmutableDictionary
from _StrictComparator import _StrictComparator
from _UnaryComparator import _UnaryComparator
from fractions import Fraction



def tabulate_well_formedness_violations_in_expr(expr, allow_empty_containers = True):
    r'''.. versionadded:: 1.1

    Tabulate well-formedness violations in `expr`::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")
        abjad> staff[1].written_duration = Duration(1, 4)
        abjad> spannertools.BeamSpanner(staff[:])
        BeamSpanner(c'8, d'4, e'8, f'8)
        abjad> f(staff)
        \new Staff {
            c'8 [
            d'4
            e'8
            f'8 ]
        }

    ::

        abjad> componenttools.tabulate_well_formedness_violations_in_expr(staff)
        1 /  4 beamed quarter note
        0 /  1 discontiguous spanner
        0 /  5 duplicate i d
        0 /  1 empty container
        0 /  0 intermarked hairpin
        0 /  0 misdurated measure
        0 /  0 misfilled measure
        0 /  4 mispitched tie
        0 /  4 misrepresented flag
        0 /  5 missing parent
        0 /  0 nested measure
        0 /  0 overlapping beam
        0 /  0 overlapping glissando
        0 /  0 overlapping octavation
        0 /  0 short hairpin

    Beamed quarter notes are not well formed.
    '''
    from abjad import checks as _checks

    for key, value in sorted(vars(_checks).items()):
        checker = value()
        checker.report(expr)



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad import Fraction


def test_BoundedInterval_split_at_rational_01():
    '''BoundedInterval.split_at_rational returns a new BoundedInterval instance.'''
    i1 = BoundedInterval(3, 23)
    splits = i1.split_at_rational(13)
    assert splits[0].signature == (3, 13)
    assert splits[1].signature == (13, 23)

def test_BoundedInterval_split_at_rational_02():
    '''A split point at or outside the BoundedInterval bounds returns the original BoundedInterval.'''
    i1 = BoundedInterval(3, 23)
    splits = i1.split_at_rational(3)
    assert i1 == splits[0]
    splits = i1.split_at_rational(23)
    assert i1 == splits[0]
    splits = i1.split_at_rational(-1000)
    assert i1 == splits[0]
    splits = i1.split_at_rational(1000)
    assert i1 == splits[0]

def test_BoundedInterval_split_at_rational_03():
    '''BoundedIntervals can be split by Fractions.'''
    i1 = BoundedInterval(3, 23)
    splits = i1.split_at_rational(Fraction(46, 13))
    assert splits[0].signature == (3, Fraction(46, 13))
    assert splits[1].signature == (Fraction(46, 13), 23)



class _Flageolet(object):
    '''Abjad model of both natural and artificial harmonics.
    Abstract base class.
    '''

    __slots__ = ()

    ### PUBLIC ATTRIBUTES ###

    @property
    def suono_reale(self):
        '''Actual sound of the harmonic when played.
        '''
        raise Exception('Not Implemented')





from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad import Fraction


def test_BoundedInterval_shift_by_rational_01():
    '''BoundedInterval.shift_by_rational returns a new BoundedInterval instance unless offset is zero.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_by_rational(1)
    assert i1 != i2
    i2 = i1.shift_by_rational(0)
    assert i1 == i2

def test_BoundedInterval_shift_by_rational_02():
    '''BoundedIntervals can be shifted by ints.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_by_rational(2)
    assert i2.low == 5
    assert i2.magnitude == i1.magnitude

def test_BoundedInterval_shift_by_rational_03():
    '''BoundedIntervals can be shifted by fractions.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_by_rational(Fraction(1, 5))
    assert i2.low == Fraction(16, 5)
    assert i2.magnitude == i1.magnitude



def negate_absolute_value_of_sequence_elements_at_indices(sequence, indices):
    '''.. versionadded:: 1.1

    Negate the absolute value of `sequence` elements at `indices`::

        abjad> from abjad.tools import sequencetools

    ::

        abjad> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]

    ::

        abjad> sequencetools.negate_sequence_elements_at_indices(sequence, [0, 1, 2])
        [-1, -2, -3, 4, 5, -6, -7, -8, -9, -10]

    Return newly constructed list.

    .. versionchanged:: 2.0
        renamed ``sequencetools.negate_elements_at_indices_absolutely()`` to
        ``sequencetools.negate_absolute_value_of_sequence_elements_at_indices()``.
    '''

    result = []

    for i, element in enumerate(sequence):
        if (i in indices):
            result.append(-abs(element))
        else:
            result.append(element)

    return result



from abjad import *


def test_componenttools_partition_components_once_by_durations_in_seconds_exactly_without_overhang_01():
    '''Fill one group exactly.
    Do not read durations cyclically.
    Do not return overhang group.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_nontied_pitched_components_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context = Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    groups = \
        componenttools.partition_components_once_by_durations_in_seconds_exactly_without_overhang(
        t.leaves, [1.5])

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(groups) == 1
    assert groups[0] == list(t.leaves[:3])



from abjad.tools.lilyfiletools._BlockNonattributed import _BlockNonattributed


class BookpartBlock(_BlockNonattributed):
    r'''.. versionadded:: 2.0

    Abjad model of LilyPond input file bookpart block.
    '''

    def __init__(self):
        _BlockNonattributed.__init__(self)
        self._escaped_name = r'\bookpart'



from abjad.exceptions import MissingInstrumentError
from abjad.tools import pitchtools
from abjad.tools import voicetools
from abjad.tools.contexttools.get_effective_instrument import get_effective_instrument


def iterate_notes_and_chords_in_expr_outside_traditional_instrument_ranges(expr):
    '''.. versionadded:: 2.0

    Iterate notes and chords in `expr` outside traditional instrument ranges::

        abjad> staff = Staff("c'8 r8 <d fs>8 r8")
        abjad> instrumenttools.Violin()(staff)
        Violin('Violin', 'Vn.')(Staff{4})

    ::

        abjad> for note_or_chord in instrumenttools.iterate_notes_and_chords_in_expr_outside_traditional_instrument_ranges(staff):
        ...   note_or_chord
        Chord('<d fs>8')

    Return generator.
    '''
    from abjad.tools import leaftools

    for note_or_chord in leaftools.iterate_notes_and_chords_forward_in_expr(expr):
        instrument = get_effective_instrument(note_or_chord)
        if instrument is None:
            raise MissingInstrumentError
        if note_or_chord not in instrument.traditional_range:
            yield note_or_chord



from abjad import *


def test_MarkupCommand_is_braced_01():
    mc = markuptools.MarkupCommand('rounded-box', None, ['one', 'two', 'three'])
    assert mc.is_braced == True


def test_MarkupCommand_is_braced_02():
    mc = markuptools.MarkupCommand('combine', None, ['one', 'two', 'three'], is_braced = False)
    assert mc.is_braced == False


def test_MarkupCommand_is_braced_03():
    mc = markuptools.MarkupCommand('concat', None, ['one', 'two', 'three'], is_braced = True)
    assert mc.is_braced == True




from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree___eq___01():
    a = IntervalTree(_make_test_intervals())
    b = IntervalTree(_make_test_intervals())
    assert a == b


def test_IntervalTree___eq___02():
    a = IntervalTree(_make_test_intervals())
    b = IntervalTree(_make_test_intervals()[:-1])
    assert a != b


def test_IntervalTree___eq___03():
    a = IntervalTree([])
    b = IntervalTree([])
    assert a == b




from abjad.tools.containertools.Container import Container
from abjad.tools.notetools.Note import Note
from abjad.tools.resttools.Rest import Rest
from abjad.exceptions import AssignabilityError
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools.tuplettools.FixedDurationTuplet import FixedDurationTuplet
import math


def make_tuplet_from_proportions_and_pair(l, (n, d), together = False):
    '''Divide `(n, d)` according to `l`.

    Where no prolation is necessary, return container. ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1], (7, 16))
        {c'4..}

    Where prolation is necessary, return fixed-duration tuplet. ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1, 2], (7, 16))
        FixedDurationTuplet(7/16, [c'8, c'4])

    ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1, 2, 4], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4])

    ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1, 2, 4, 1], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16])

    ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1, 2, 4, 1, 2], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16, c'8])

    ::

        abjad> tuplettools.make_tuplet_from_proportions_and_pair([1, 2, 4, 1, 2, 4], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16, c'8, c'4])

    .. note:: function accepts a pair rather than a rational.

    .. note:: function interprets `d` as tuplet denominator.

    .. versionchanged:: 2.0
        renamed ``divide.pair()`` to
        ``tuplettools.make_tuplet_from_proportions_and_pair()``.
    '''


    duration = (n, d)

    if len(l) == 0:
        raise ValueError('must divide list l of poisitive length.')

    if len(l) == 1:
        if 0 < l[0]:
            try:
                return Container([Note(0, duration)])
            except AssignabilityError:
                return Container(notetools.make_notes(0, duration))
        elif l[0] < 0:
            try:
                return Container([Rest(duration)])
            except AssignabilityError:
                return Container(resttools.make_rests(duration))
        else:
            raise ValueError('no divide zero values.')

    if 1 < len(l):
        exponent = int(math.log(mathtools.weight(l), 2) - math.log(n, 2))
        denominator = int(d * 2 ** exponent)
        music = []
        for x in l:
            if not x:
                raise ValueError('no divide zero values.')
            if 0 < x:
                try:
                    music.append(Note(0, (x, denominator)))
                except AssignabilityError:
                    music.extend(notetools.make_notes(0, (x, denominator)))
            else:
                music.append(Rest((-x, denominator)))
        return FixedDurationTuplet(duration, music)



from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad import Fraction


def scale_interval_magnitudes_by_rational(intervals, rational):
    '''Scale the magnitude of each interval in `intervals` by
    `rational`, maintaining their low offsets ::

        abjad> from abjad.tools import intervaltreetools
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> from abjad.tools.intervaltreetools import IntervalTree

    ::

        abjad> a = BoundedInterval(-1, 3)
        abjad> b = BoundedInterval(6, 12)
        abjad> c = BoundedInterval(9, 16)
        abjad> tree = IntervalTree([a, b, c])
        abjad> intervaltreetools.scale_interval_magnitudes_by_rational(tree, Fraction(6, 5))
        IntervalTree([
            BoundedInterval(Offset(-1, 1), Offset(19, 5), {}),
            BoundedInterval(Offset(6, 1), Offset(66, 5), {}),
            BoundedInterval(Offset(9, 1), Offset(87, 5), {})
        ])

    Return interval tree.
    '''

    assert isinstance(rational, (int, Fraction)) and 0 < rational
    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree or rational == 1:
        return tree

    return IntervalTree([
        x.scale_by_rational(rational) for x in tree
    ])



from abjad import *
import py.test


def test_SchemeNumber___setattr___01():
    '''Scheme numbers are immutable.
    '''

    scheme_number = schemetools.SchemeNumber(-1)
    assert py.test.raises(AttributeError, "scheme_number.foo = 'bar'")



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction
import py.test


def test_intervaltreetools_split_intervals_at_rationals_01():
    splits = [-1, 16]
    a = BoundedInterval(0, 10)
    b = BoundedInterval(5, 15)
    tree = IntervalTree([a, b])
    split = split_intervals_at_rationals(tree, splits)
    assert tree[:] == split[:]







from abjad.tools.sequencetools import iterate_sequence_pairwise_strict
from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad.tools.intervaltreetools.get_all_unique_bounds_in_intervals import get_all_unique_bounds_in_intervals
from abjad.tools.intervaltreetools.split_intervals_at_rationals import split_intervals_at_rationals


def compute_depth_of_intervals_in_interval(intervals, interval):
    '''Compute a tree whose intervals represent the depth (level of overlap)
    in each boundary pair of `intervals`, cropped within `interval`::

        abjad> from abjad.tools.intervaltreetools import *
        abjad> a = BoundedInterval(0, 3)
        abjad> b = BoundedInterval(6, 12)
        abjad> c = BoundedInterval(9, 15)
        abjad> tree = IntervalTree([a, b, c])
        abjad> d = BoundedInterval(-1, 16)
        abjad> compute_depth_of_intervals_in_interval(tree, d)
        IntervalTree([
            BoundedInterval(Offset(-1, 1), Offset(0, 1), {'depth': 0}),
            BoundedInterval(Offset(0, 1), Offset(3, 1), {'depth': 1}),
            BoundedInterval(Offset(3, 1), Offset(6, 1), {'depth': 0}),
            BoundedInterval(Offset(6, 1), Offset(9, 1), {'depth': 1}),
            BoundedInterval(Offset(9, 1), Offset(12, 1), {'depth': 2}),
            BoundedInterval(Offset(12, 1), Offset(15, 1), {'depth': 1}),
            BoundedInterval(Offset(15, 1), Offset(16, 1), {'depth': 0})
        ])

    Return interval tree.
    '''

    assert all_are_intervals_or_trees_or_empty(intervals)
    assert isinstance(interval, BoundedInterval)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)

    if interval.high <= tree.low or tree.high <= interval.low:
        return IntervalTree([BoundedInterval(interval.low, interval.high, {'depth': 0})])
    else:
        bounds = list(get_all_unique_bounds_in_intervals(tree))
        if interval.low < tree.low:
            bounds.insert(0, interval.low)
        elif tree.low < interval.low:
            bounds = filter(lambda x: interval.low <= x, bounds)
            bounds.insert(0, interval.low)
        if tree.high < interval.high:
            bounds.append(interval.high)
        elif interval.high < tree.high:
            bounds = filter(lambda x: x <= interval.high, bounds)
            bounds.append(interval.high)
        bounds = sorted(list(set(bounds)))

    intervals = []
#   for i in range(len(bounds) - 1):
    for pair in iterate_sequence_pairwise_strict(bounds):
        target = BoundedInterval(pair[0], pair[1], {})
        found = tree.find_intervals_intersecting_or_tangent_to_interval(target)
        if found:
            target['depth'] = len(filter( \
                lambda x: not x.low == target.high and not x.high == target.low, found))
        else:
            target['depth'] = 0
        intervals.append(target)

    return IntervalTree(intervals)



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree_find_intervals_starting_before_offset_01():
    blocks = _make_test_intervals()
    target_offset = 0
    expected_payloads = ()
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_before_offset_02():
    blocks = _make_test_intervals()
    target_offset = 9
    expected_payloads = ('a', 'b', 'c', 'd',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_before_offset_03():
    blocks = _make_test_intervals()
    target_offset = 14
    expected_payloads = ('a', 'b', 'c', 'd',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_before_offset_04():
    blocks = _make_test_intervals()
    target_offset = 19
    expected_payloads = ('a', 'b', 'c', 'd', 'e', 'f', 'g',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_before_offset_05():
    blocks = _make_test_intervals()
    target_offset = 26
    expected_payloads = ('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_before_offset_06():
    blocks = _make_test_intervals()
    target_offset = 30
    expected_payloads = ('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_before_offset(target_offset)
        assert expected_blocks == actual_blocks



from abjad import *


def test_TrillSpanner___init___01():
    '''Init empty trill spanner.
    '''

    trill = spannertools.TrillSpanner()
    assert isinstance(trill, spannertools.TrillSpanner)






def _get_module_members(module_path, klass='class'):
   '''Get members in module.'''

   f = open(module_path, 'r')
   lines = f.readlines()
   f.close()
   result = [ ]
   for line in lines:
      if line.startswith(klass):
         attr_name = line.split(' ')[1].split('(')[0]
         result.append(attr_name)
   return result



from abjad import *
from abjad.tools import durationtools


def test_durationtools_yield_all_positive_rationals_in_cantor_diagonalized_order_uniquely_01():

    generator = durationtools.yield_all_positive_rationals_in_cantor_diagonalized_order_uniquely()

    assert generator.next() == Fraction(1, 1)
    assert generator.next() == Fraction(2, 1)
    assert generator.next() == Fraction(1, 2)
    assert generator.next() == Fraction(1, 3)
    assert generator.next() == Fraction(3, 1)
    assert generator.next() == Fraction(4, 1)
    assert generator.next() == Fraction(3, 2)
    assert generator.next() == Fraction(2, 3)
    assert generator.next() == Fraction(1, 4)
    assert generator.next() == Fraction(1, 5)
    assert generator.next() == Fraction(5, 1)
    assert generator.next() == Fraction(6, 1)
    assert generator.next() == Fraction(5, 2)
    assert generator.next() == Fraction(4, 3)
    assert generator.next() == Fraction(3, 4)
    assert generator.next() == Fraction(2, 5)




from abjad.tools import contexttools
from abjad.tools.instrumenttools._DoubleReedInstrument import _DoubleReedInstrument


class Bassoon(_DoubleReedInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the bassoon::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")
        abjad> contexttools.ClefMark('bass')(staff)
        ClefMark('bass')(Staff{4})

    ::

        abjad> instrumenttools.Bassoon()(staff)
        Bassoon('Bassoon', 'Bsn.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \clef "bass"
            \set Staff.instrumentName = \markup { Bassoon }
            \set Staff.shortInstrumentName = \markup { Bsn. }
            c'8
            d'8
            e'8
            f'8
        }

    The bassoon targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Bassoon', short_instrument_name = 'Bsn.', target_context = None):
        _DoubleReedInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.primary_clefs = [contexttools.ClefMark('bass')]
        self.all_clefs = [contexttools.ClefMark('bass'), contexttools.ClefMark('tenor')]
        self.traditional_range = (-26, 15)



class _BlockNonattributed(list):
    r'''Abjad model of the LilyPond input file blocks with no attributes.
    '''

    ### OVERLOADS ###

    def __repr__(self):
        if not len(self):
            return '%s()' % type(self).__name__
        else:
            return '%s(%s)' % (type(self).__name__, len(self))

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_pieces(self):
        result = []
        if not len(self):
            result.append(r'%s {}' % self._escaped_name)
        else:
            result.append(r'%s {' % self._escaped_name)
            for x in self:
                result.extend(['\t' + piece for piece in x._format_pieces])
            result.append('}')
        return result

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        return '\n'.join(self._format_pieces)



from abjad.core import _Immutable
from abjad.core import _StrictComparator


class SchemeNumber(_Immutable):
    '''Abjad model of Scheme number::

        abjad> schemetools.SchemeNumber(1.1)
        SchemeNumber(1.1...)

    Scheme numbers are immutable.
    '''

    __slots__ = ('number',)

    def __new__(klass, number):
        assert type(number) in (float, int)
        self = object.__new__(klass)
        object.__setattr__(self, 'number', number)
        return self

    def __getnewargs__(self):
        return (self.number, )

    ### OVERLOADS ###

    def __eq__(self, arg):
        if isinstance(arg, SchemeNumber):
            return self.number == arg.number
        return False

    def __float__(self):
        return float(self.number)

    def __int__(self):
        return int(self.number)

    def __repr__(self):
        return "%s(%r)" % (type(self).__name__, self.number)

    def __str__(self):
        return self.format

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme number::

            abjad> scheme_number = schemetools.SchemeNumber(1.1)
            abjad> scheme_number.format
            '#1.1'

        Return string.
        '''
        return '#%s' % self.number



class _UnaryComparator(object):
    '''Mix-in base class to confer unary comparison behavior to any custom class.
    '''

    __slots__ = ('_comparison_attribute', '_format_string')

    ### OVERLOADS ###

    def __eq__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute == arg._comparison_attribute

    def __ge__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute >= arg._comparison_attribute

    def __gt__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute > arg._comparison_attribute

    def __le__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute <= arg._comparison_attribute

    def __lt__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute < arg._comparison_attribute

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self._comparison_attribute)

    ### PRIVATE METHODS ###

    def _massage_comparison_arg(self, arg):
        if not isinstance(arg, type(self)):
            try:
                arg = type(self)(arg)
            except (ValueError, TypeError):
                return False
        return arg




from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree___getslice____01():
    intervals = _make_test_intervals()
    tree = IntervalTree(intervals)
    assert tuple(intervals[:]) == tree[:]
    assert tuple(intervals[:2]) == tree[:2]
    assert tuple(intervals[-2:]) == tree[-2:]
    assert tuple(intervals[1:3]) == tree[1:3]



from abjad import *


def test_DecrescendoSpanner___init___01():
    '''Init empty decrescendo spanner.
    '''

    decrescendo = spannertools.DecrescendoSpanner()
    assert isinstance(decrescendo, spannertools.DecrescendoSpanner)


def test_DecrescendoSpanner___init___02():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    spannertools.DecrescendoSpanner(staff[:4])

    r'''
    \new Staff {
        c'8 \>
        d'8
        e'8
        f'8 \!
        g'2
    }
    '''

    assert staff.format == "\\new Staff {\n\tc'8 \\>\n\td'8\n\te'8\n\tf'8 \\!\n\tg'2\n}"






from abjad import *


def test_MarkupCommand___eq___01():

    a = markuptools.MarkupCommand('box', None, None)
    b = markuptools.MarkupCommand('box', ['hello'], None)
    c = markuptools.MarkupCommand('box', None, ['hello'])
    d = markuptools.MarkupCommand('box', ['hello'], ['hello'])
    e = markuptools.MarkupCommand('sox', None, None)

    assert a != b != c != d != e

def test_MarkupCommand___eq___02():

    a = markuptools.MarkupCommand('box', None, None)
    b = markuptools.MarkupCommand('box', None, None)

    assert a == b



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad import Fraction
import py.test


def test_BoundedInterval_scale_to_rational_01():
    '''BoundedInterval.scale_to_rational returns a new BoundedInterval instance unless offset is old magnitude.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_to_rational(21)
    assert i1 != i2
    i2 = i1.scale_to_rational(20)
    assert i1 == i2

def test_BoundedInterval_scale_to_rational_02():
    '''BoundedInterval magnitudes can be scaled to int offsets.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_to_rational(10)
    assert i1.low == i2.low
    assert i2.magnitude == 10

def test_BoundedInterval_scale_to_rational_03():
    '''BoundedInterval magnitudes can be scaled to Fractional offsets.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_to_rational(Fraction(2, 5))
    assert i1.low == i2.low
    assert i2.magnitude == Fraction(2, 5)

def test_BoundedInterval_scale_to_rational_04():
    '''BoundedInterval magnitudes cannot be scaled to zero.'''
    i1 = BoundedInterval(3, 23)
    py.test.raises(AssertionError,
        'i1.scale_to_rational(0)')

def test_BoundedInterval_scale_to_rational_05():
    '''BoundedInterval magnitudes cannot be scaled to negative offsets.'''
    i1 = BoundedInterval(3, 23)
    py.test.raises(AssertionError,
        'i2 = i1.scale_to_rational(-1)')




from abjad.tools.intervaltreetools import BoundedInterval


def _make_test_intervals():

    return [
        BoundedInterval(0, 3,   {'a': 1}),
        BoundedInterval(5, 13,  {'b': 2}),
        BoundedInterval(6, 10,  {'c': 3}),
        BoundedInterval(8, 9,   {'d': 4}),
        BoundedInterval(15, 23, {'e': 5}),
        BoundedInterval(16, 21, {'f': 6}),
        BoundedInterval(17, 19, {'g': 7}),
        BoundedInterval(19, 20, {'h': 8}),
        BoundedInterval(25, 30, {'i': 9}),
        BoundedInterval(26, 29, {'j': 10}),
        BoundedInterval(32, 34, {'k': 11}),
        BoundedInterval(34, 37, {'l': 12}),
    ]



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
import py.test


def test_intervaltreetools_compute_logical_not_of_intervals_in_interval_01():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(1, 14)
    logic = compute_logical_not_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == [(3, 6)]

def test_intervaltreetools_compute_logical_not_of_intervals_in_interval_02():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(-1, 16)
    logic = compute_logical_not_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == [(-1, 0), (3, 6), (15, 16)]

def test_intervaltreetools_compute_logical_not_of_intervals_in_interval_03():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(2001, 2010)
    logic = compute_logical_not_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == [(2001, 2010)]



class _LilyPondObjectProxy(object):
    '''.. versionadded:: 2.0

    Shared LilyPond grob proxy and LilyPond context proxy functionality.
    '''

    def __init__(self, **kwargs):
        for key, value in kwargs.iteritems( ):
            setattr(self, key, value)

    ### OVERLOADS ###

    def __copy__(self):
        return eval(repr(self))

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            return self._get_attribute_pairs( ) == arg._get_attribute_pairs( )
        return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        #body_string = ' '
        body_string = ''
        skeleton_strings = self._get_skeleton_strings( )
        if skeleton_strings:
            body_string = ', '.join(skeleton_strings)
        return '%s(%s)' % (self.__class__.__name__, body_string)

    ### PRIVATE METHODS ###

    def _get_attribute_pairs(self):
        return tuple(vars(self).iteritems( ))

    def _get_skeleton_strings(self):
        result = [ ]
        for attribute_name, attribute_value in self._get_attribute_pairs( ):
            result.append('%s = %s' % (attribute_name, repr(attribute_value)))
        return result




from abjad import *


def test_pitchtools_PitchRange___init___01():
    '''Init infinite pitch range.'''

    pr = pitchtools.PitchRange()
    assert pr._start is None
    assert pr._stop is None


def test_pitchtools_PitchRange___init___02():
    '''Init stop-specified pitch range.'''

    pr = pitchtools.PitchRange((-39, 'inclusive'), None)
    assert pr._start == (pitchtools.NamedChromaticPitch(-39), 'inclusive')
    assert pr._stop is None

    pr = pitchtools.PitchRange((-39, 'exclusive'), None)
    assert pr._start == (pitchtools.NamedChromaticPitch(-39), 'exclusive')
    assert pr._stop is None


def test_pitchtools_PitchRange___init___03():
    '''Init start-specified pitch range.'''

    pr = pitchtools.PitchRange(None, (48, 'inclusive'))
    assert pr._start is None
    assert pr._stop == (pitchtools.NamedChromaticPitch(48), 'inclusive')

    pr = pitchtools.PitchRange(None, (48, 'exclusive'))
    assert pr._start is None
    assert pr._stop == (pitchtools.NamedChromaticPitch(48), 'exclusive')


def test_pitchtools_PitchRange___init___04():
    '''Init start- and stop-specified pitch range.'''

    pr = pitchtools.PitchRange((-39, 'inclusive'), (48, 'inclusive'))
    assert pr._start == (pitchtools.NamedChromaticPitch(-39), 'inclusive')
    assert pr._stop == (pitchtools.NamedChromaticPitch(48), 'inclusive')


def test_pitchtools_PitchRange___init___05():
    '''Short-form init with only integers.'''

    pr = pitchtools.PitchRange(-39, 48)
    assert pr._start == (pitchtools.NamedChromaticPitch(-39), 'inclusive')
    assert pr._stop == (pitchtools.NamedChromaticPitch(48), 'inclusive')


def test_pitchtools_PitchRange___init___06():
    '''Init pitch range from other pitch range.
    '''

    pitch_range_1 = pitchtools.PitchRange(-39, 48)
    pitch_range_2 = pitchtools.PitchRange(pitch_range_1)

    assert isinstance(pitch_range_1, pitchtools.PitchRange)
    assert isinstance(pitch_range_2, pitchtools.PitchRange)
    assert pitch_range_1 == pitch_range_2
    assert pitch_range_1 is not pitch_range_2


def test_pitchtools_PitchRange___init___07():
    '''Init pitch range from pair.
    '''

    pitch_range_1 = pitchtools.PitchRange(-39, 48)
    pitch_range_2 = pitchtools.PitchRange((-39, 48))

    assert pitch_range_1 == pitch_range_2




from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_intervaltreetools_explode_intervals_into_n_trees_heuristically_01():
    tree = IntervalTree(_make_test_intervals())
    n = 1
    result = explode_intervals_into_n_trees_heuristically(tree, n)
    assert result == \
    [IntervalTree([ \
        BoundedInterval(0, 3, {'a': 1}), \
        BoundedInterval(5, 13, {'b': 2}), \
        BoundedInterval(6, 10, {'c': 3}), \
        BoundedInterval(8, 9, {'d': 4}), \
        BoundedInterval(15, 23, {'e': 5}), \
        BoundedInterval(16, 21, {'f': 6}), \
        BoundedInterval(17, 19, {'g': 7}), \
        BoundedInterval(19, 20, {'h': 8}), \
        BoundedInterval(25, 30, {'i': 9}), \
        BoundedInterval(26, 29, {'j': 10}), \
        BoundedInterval(32, 34, {'k': 11}), \
        BoundedInterval(34, 37, {'l': 12}) \
    ])]
    if n == calculate_min_mean_and_max_depth_of_intervals(tree)[2]:
        assert all([all_intervals_are_nonoverlapping(x) for x in result])

def test_intervaltreetools_explode_intervals_into_n_trees_heuristically_02():
    tree = IntervalTree(_make_test_intervals())
    n = 2
    result = explode_intervals_into_n_trees_heuristically(tree, n)
    assert result == \
    [IntervalTree([
        BoundedInterval(0, 3, {'a': 1}),
        BoundedInterval(6, 10, {'c': 3}),
        BoundedInterval(8, 9, {'d': 4}),
        BoundedInterval(15, 23, {'e': 5}),
        BoundedInterval(17, 19, {'g': 7}),
        BoundedInterval(19, 20, {'h': 8}),
        BoundedInterval(26, 29, {'j': 10})
    ]), IntervalTree([
        BoundedInterval(5, 13, {'b': 2}),
        BoundedInterval(16, 21, {'f': 6}),
        BoundedInterval(25, 30, {'i': 9}),
        BoundedInterval(32, 34, {'k': 11}),
        BoundedInterval(34, 37, {'l': 12})
    ])]
    if n == calculate_min_mean_and_max_depth_of_intervals(tree)[2]:
        assert all([all_intervals_are_nonoverlapping(x) for x in result])


def test_intervaltreetools_explode_intervals_into_n_trees_heuristically_03():
    tree = IntervalTree(_make_test_intervals())
    n = 3
    result = explode_intervals_into_n_trees_heuristically(tree, n)
    assert result == \
    [IntervalTree([
        BoundedInterval(0, 3, {'a': 1}),
        BoundedInterval(8, 9, {'d': 4}),
        BoundedInterval(15, 23, {'e': 5}),
        BoundedInterval(32, 34, {'k': 11}),
        BoundedInterval(34, 37, {'l': 12})
    ]), IntervalTree([
        BoundedInterval(5, 13, {'b': 2}),
        BoundedInterval(17, 19, {'g': 7}),
        BoundedInterval(19, 20, {'h': 8}),
        BoundedInterval(26, 29, {'j': 10})
    ]), IntervalTree([
        BoundedInterval(6, 10, {'c': 3}),
        BoundedInterval(16, 21, {'f': 6}),
        BoundedInterval(25, 30, {'i': 9})
    ])]
    if n == calculate_min_mean_and_max_depth_of_intervals(tree)[2]:
        assert all([all_intervals_are_nonoverlapping(x) for x in result])



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction
import py.test


def test_intervaltreetools_scale_aggregate_magnitude_to_rational_01():
    a = BoundedInterval(Fraction(-1, 2), 1)
    b = BoundedInterval(2, Fraction(7, 3))
    tree = IntervalTree([a, b])
    rational = 3
    result = scale_aggregate_magnitude_to_rational(tree, rational)
    assert result.magnitude == 3
    assert [x.signature for x in result] == \
        [(Fraction(-1, 2), Fraction(37, 34)), (Fraction(73, 34), Fraction(5, 2))]

def test_intervaltreetools_scale_aggregate_magnitude_to_rational_02():
    a = BoundedInterval(Fraction(-1, 2), 1)
    b = BoundedInterval(2, Fraction(7, 3))
    tree = IntervalTree([a, b])
    rational = -1
    py.test.raises(AssertionError,
        "result = scale_aggregate_magnitude_to_rational(tree, rational)")

def test_intervaltreetools_scale_aggregate_magnitude_to_rational_03():
    a = BoundedInterval(Fraction(-1, 2), 1)
    b = BoundedInterval(2, Fraction(7, 3))
    tree = IntervalTree([a, b])
    rational = 0
    py.test.raises(AssertionError,
        "result = scale_aggregate_magnitude_to_rational(tree, rational)")



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval


def test_BoundedInterval_magnitude_01():
    '''BoundedInterval magnitude is the high minus the low offset.'''
    i = BoundedInterval(3, 23)
    assert (23 - 3) == i.magnitude



from abjad.cfg.cfg import ABJADVERSIONFILE


def get_abjad_revision_string():
    '''.. versionadded:: 2.0

    Get Abjad revision string::

        abjad> configurationtools.get_abjad_revision_string() # doctest: +SKIP
        '4392'

    Return string.
    '''

    return file(ABJADVERSIONFILE, 'r').read().strip()



from abjad.tools.instrumenttools._ReedInstrument import _ReedInstrument


class _SingleReedInstrument(_ReedInstrument):
    '''.. versionadded:: 2.0

    Abjad model of single-reed instruments.
    '''

    pass



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad import Fraction


def test_BoundedInterval_shift_to_rational_01():
    '''BoundedInterval.shift_to_rational returns a new BoundedInterval instance unless offset is old low offset.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_to_rational(-1000)
    assert i1 != i2
    i2 = i1.shift_to_rational(3)
    assert i1 == i2

def test_BoundedInterval_shift_to_rational_02():
    '''BoundedIntervals can be shifted by ints.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_to_rational(-1)
    assert i2.low == -1
    assert i2.magnitude == i1.magnitude

def test_BoundedInterval_shift_to_rational_03():
    '''BoundedIntervals can be shifted by Fractions.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.shift_to_rational(Fraction(-51, 3))
    assert i2.low == Fraction(-51, 3)
    assert i2.magnitude == i1.magnitude



from _get_module_members import _get_module_members
import os


def _get_title_type_members(source_full_path):

   #print source_full_path
   # starts as '/Users/foo/bar/abjad/trunk/abjad/tools/sequencetools/do_stuff.py'

   parts = [ ]
   for part in reversed(source_full_path.split(os.sep)):
      if not part == 'abjad':
         parts.insert(0, part)
      else:
         parts.insert(0, part)
         break

   # ends as 'abjad.tools.sequencetools.do_stuff'
   parts = '.'.join(parts)
   parts = parts[:-3]
   #print parts

   # name is something like 'apply_octavation' or 'Accidental'
   name = parts.split('.')[-1]
   #print name

   # module is in one of the tools packages
   if parts.startswith('abjad.tools.'):
      page_title = parts[12:]
      #print 'PAGE TITLE is %s' % page_title
      # if source file in tools package implements public helper function
      if name[0].islower():
         auto_type = 'autofunction'
         functions = _get_module_members(source_full_path, 'def')
      # if source file in tools package implements public class
      else:
         page_title = page_title.split('.')
         page_title = page_title[:-1]
         page_title = '.'.join(page_title)
         # page title is now something like pitchtools.NamedChromaticPitchClass
         auto_type = 'autoclass'
         functions = _get_module_members(source_full_path, 'class')
      public_functions = [x for x in functions if not x.startswith('_')]
      members = public_functions
      # check if file defines only private _get_measure_from_component(), for example
      if not members:
         #print 'NOT rendering %s ...' % page_title
         page_title = None

   # or is the exceptions module
   elif 'exceptions' in source_full_path:
      page_title = 'exceptions'
      auto_type = 'autoexception'
      members = _get_module_members(source_full_path, 'class')

   # or is a class file
   elif _get_module_members(source_full_path, 'class'):
      members = _get_module_members(source_full_path, 'class')
      if 1 < len(members):
         raise ValueError('%s defines more than 1 public class!' %
            source_full_path)
      page_title = members[0]
      auto_type = 'autoclass'

   # or contains public functions, like dfs(), outside of tools packages
   elif _get_module_members(source_full_path, 'def'):
      raise ValueError('no public helpers outside of tools packages!')

   # or else contains only non-documenting helper functions
   else:
      raise ValueError('unkonwn type of module content: %s.' % source_full_path)

   return page_title, auto_type, members



from abjad import *
import py.test


def test_SchemeBoolean___setattr___01():
    '''Scheme booleans are immutable.
    '''

    scheme_boolean = schemetools.SchemeBoolean(False)
    assert py.test.raises(AttributeError, "scheme_boolean.foo = 'bar'")



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag
from abjad.tools.pitchtools import Accidental
from abjad.tools.pitchtools import NamedChromaticPitch
from abjad.tools.pitchtools import apply_accidental_to_named_chromatic_pitch


def _pitch_node_to_named_pitch(node):
    assert _all_are_nodes_with_tag([node], 'note')
    step = node.find('step').text
    octave = int(node.find('octave').text)
    pitch = NamedChromaticPitch(step, octave)
    if node.find('alter') is not None:
        return apply_accidental_to_named_chromatic_pitch(
            pitch, Accidental(int(node.find('alter').text)))
    return pitch



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from collections import Iterable


def all_are_intervals_or_trees_or_empty(input):
    '''Recursively test if all elements of `input` are
    BoundedIntervals or IntervalTrees.
    An empty result also return as True.
    '''

    def recurse(x):
        if isinstance(x, Iterable) and \
        not isinstance(x, (basestring, IntervalTree, BoundedInterval)):
            return [a for i in x for a in recurse(i)]
        else:
            return [x]

    return all([isinstance(x, (BoundedInterval, IntervalTree)) \
        for x in recurse(input)])



from xml.etree import ElementTree


def _all_are_nodes_with_tag(nodes, tag):
    assert len(nodes)
    if all([ElementTree.iselement(node) for node in nodes]) and \
        all([node.tag == tag for node in nodes]):
        return True
    return False



from abjad.tools.durationtools import Offset
from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_BoundedInterval_centroid_01():
    intervals = _make_test_intervals()
    for interval in intervals:
        assert interval.centroid == Offset(interval.low + interval.high, 2)



from abjad import *


def test_SchemeString___init___01():

    t = Note(0, (1, 16))
    t.override.stem.stroke_style = schemetools.SchemeString('grace')

    r'''
    \once \override Stem #'stroke-style = #"grace"
    c'16
    '''

    assert t.format == '\\once \\override Stem #\'stroke-style = #"grace"\nc\'16'



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad.tools.intervaltreetools.compute_depth_of_intervals_in_interval import compute_depth_of_intervals_in_interval
from abjad.tools.intervaltreetools.fuse_tangent_or_overlapping_intervals import fuse_tangent_or_overlapping_intervals


def compute_logical_or_of_intervals_in_interval(intervals, interval):
    '''Compute the logical OR of a collection of intervals,
    cropped within `interval`.
    '''

    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree:
        return tree

    depth_tree = compute_depth_of_intervals_in_interval(tree, interval)
    logic_tree = IntervalTree(filter(lambda x: 1 <= x['depth'], depth_tree))

    return logic_tree

#   if not logic_tree:
#      return logic_tree
#   return fuse_tangent_or_overlapping_intervals(logic_tree)



def _withdraw_component_from_attached_spanners(component):
    '''Withdraw `component` from all attached spanners.
    '''

    for spanner in component.spanners:
        index = spanner.index(component)
        spanner._remove(component)






from abjad import *


def test_PaperBlock_01():

    pb = lilyfiletools.PaperBlock()
    pb.top_margin = 15
    pb.left_margin = 15

    r'''
    \paper {
        left-margin = #15
        top-margin = #15
    }
    '''

    assert pb.format == '\\paper {\n\tleft-margin = #15\n\ttop-margin = #15\n}'



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree_high_min_01():
    '''high_min returns minimum high value of all intervals in tree.'''
    blocks = _make_test_intervals()
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0))
        tree = IntervalTree(blocks)
        assert tree.high_min == 3

def test_IntervalTree_high_min_02():
    '''high_min returns None if no intervals in tree.'''
    tree = IntervalTree([])
    assert tree.high_min is None




from _get_documenting_directories import _get_documenting_directories
from abjad.cfg.cfg import ABJADPATH
import os


def _get_documenting_modules():
   '''Return a list of Abjad modules to search for
   classes and functions to document in the public API.
   '''

   modules = [ ]
   documenting_directories = _get_documenting_directories()
   for documenting_directory in documenting_directories:
      documenting_directory = os.path.join(ABJADPATH, documenting_directory)
      for current_root, dirs, files in os.walk(documenting_directory):
         if '.svn' not in current_root:
            if 'test' not in current_root:
               for file in files:
                  if file.endswith('.py'):
                     if not file.startswith('__'):
                        module = os.path.join(current_root, file)
                        modules.append(module)
   return modules
   

if __name__ == '__main__':
   modules = _get_documenting_modules()
   for x in modules:
      print x



from abjad import *
import py.test


def test_SchemeString___setattr___01():
    '''Scheme strings are immutable.
    '''

    scheme_string = schemetools.SchemeString('grace')
    assert py.test.raises(AttributeError, "scheme_string.foo = 'bar'")



from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Flute import Flute


class AltoFlute(Flute):
    r'''.. versionadded 1.1.2

    Abjad model of the alto flute::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.AltoFlute()(staff)
        AltoFlute('Alto Flute', 'Alt. Fl.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Alto Flute }
            \set Staff.shortInstrumentName = \markup { Alt. Fl. }
            c'8
            d'8
            e'8
            f'8
        }

    The alto flute targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Alto Flute', short_instrument_name = 'Alt. Fl.', target_context = None):
        Flute.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch("g")
        self.traditional_range = (-5, 31)



from abjad import *


def test_measuretools_get_nth_measure_in_expr_01():

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_nontied_pitched_components_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    assert measuretools.get_nth_measure_in_expr(staff, 0) is staff[0]
    assert measuretools.get_nth_measure_in_expr(staff, 1) is staff[1]
    assert measuretools.get_nth_measure_in_expr(staff, 2) is staff[2]


def test_measuretools_get_nth_measure_in_expr_02():

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_nontied_pitched_components_in_expr(staff)

    assert measuretools.get_nth_measure_in_expr(staff, -1) is staff[2]
    assert measuretools.get_nth_measure_in_expr(staff, -2) is staff[1]
    assert measuretools.get_nth_measure_in_expr(staff, -3) is staff[0]






from abjad.tools.componenttools._partition_components_by_durations import _partition_components_by_durations


def partition_components_cyclically_by_prolated_durations_exactly_without_overhang(
    components, prolated_durations):
    '''.. versionadded:: 1.1

    Partition `components` cyclically by `prolated_durations` exactly, without overhang.
    '''

    parts = _partition_components_by_durations('prolated', components, prolated_durations,
        fill = 'exact', cyclic = True, overhang = False)

    return parts



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction
import py.test


def test_intervaltreetools_clip_interval_magnitudes_to_range_01():
    low = None
    high = None
    tree = IntervalTree(_make_test_intervals())
    clipped = clip_interval_magnitudes_to_range(tree, low, high)
    assert clipped[:] == tree[:]
    assert sorted([x.low for x in tree]) == sorted([x.low for x in clipped])


def test_intervaltreetools_clip_interval_magnitudes_to_range_02():
    low = Fraction(3, 4)
    high = None
    tree = IntervalTree(_make_test_intervals())
    clipped = clip_interval_magnitudes_to_range(tree, low, high)
    assert all([low <= x.magnitude for x in clipped])
    assert sorted([x.low for x in tree]) == sorted([x.low for x in clipped])


def test_intervaltreetools_clip_interval_magnitudes_to_range_03():
    low = None
    high = Fraction(1, 5)
    tree = IntervalTree(_make_test_intervals())
    clipped = clip_interval_magnitudes_to_range(tree, low, high)
    assert all([x.magnitude <= high for x in clipped])
    assert sorted([x.low for x in tree]) == sorted([x.low for x in clipped])


def test_intervaltreetools_clip_interval_magnitudes_to_range_04():
    low = Fraction(1, 7)
    high = Fraction(1, 3)
    tree = IntervalTree(_make_test_intervals())
    clipped = clip_interval_magnitudes_to_range(tree, low, high)
    assert all([low <= x.magnitude <= high for x in clipped])
    assert sorted([x.low for x in tree]) == sorted([x.low for x in clipped])


def test_intervaltreetools_clip_interval_magnitudes_to_range_05():
    low = Fraction(1, 3)
    high = Fraction(1, 7)
    tree = IntervalTree(_make_test_intervals())
    py.test.raises(AssertionError, "clipped = clip_interval_magnitudes_to_range(tree, low, high)")



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag


def _note_nodes_contain_chords(nodes):
    assert _all_are_nodes_with_tag(nodes, 'note')
    if filter(lambda x: x.find('chord') is not None, nodes):
        return True
    return False



from _get_documenting_modules import _get_documenting_modules


def _get_documenting_names():
   '''Return a list of Abjad classes and functions to document in public API.
   Sort list alphabetically by name.
   Each list entry has the form of a dictionary with three items.
   The dictionary keys are 'name', 'kind' and 'module'.
   '''

   names = [ ]
   for module in _get_documenting_modules():
      f = open(module, 'r')
      lines = f.readlines()
      f.close()
      for line in lines:
         if line.startswith('def'):
            name = line.split(' ')[1].split('(')[0]
            names.append({'name': name, 'kind': 'function', 'module': module})
         elif line.startswith('class'):
            name = line.split(' ')[1].split('(')[0]
            names.append({'name': name, 'kind': 'class', 'module': module})
   names.sort(lambda x, y: cmp(x['name'], y['name']))
   for name in names[:]:
      if 'ParentageInterface' in name['name']:
         names.remove(name)
   return names

if __name__ == '__main__':
   names = _get_documenting_names()
   for name in names:
      print name



from abjad.core import _StrictComparator


class _ContainmentSignature(_StrictComparator):

    def __init__(self):
        self._root = None
        self._root_str = ''
        self._self = None
        self._score = None
        self._staff = None
        self._staffgroup = None
        self._voice = None

    ### OVERLOADS ###

    def __eq__(self, arg):
        return isinstance(arg, _ContainmentSignature) and \
            self._voice == arg._voice and \
            self._staff == arg._staff and \
            self._staffgroup == arg._staffgroup and \
            self._score == arg._score and \
            self._root == arg._root

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        str = self.__str__()
        return '<' + ' * '.join(str.split('\n')) + ' >'

    def __str__(self):
        result = []
        result.append('        root: %s (%s)' % (self._root_str, self._root))
        result.append('     score: %s' % (self._score or ''))
        result.append('staffgroup: %s' % (self._staffgroup or ''))
        result.append('     staff: %s' % (self._staff or ''))
        result.append('     voice: %s' % (self._voice or ''))
        result.append('        self: %s' % self._self)
        return '\n'.join(result)



from abjad.core import _FlexEqualityComparator
from abjad.core import _Immutable


class _PitchClass(_Immutable, _FlexEqualityComparator):
    '''.. versionadded:: 2.0

    Pitch-class base class.
    '''

    ### OVERLOADS ###

    def __hash__(self):
        return hash(repr(self))




from abjad import *


def test_instrumenttools_Clarinet_sounding_pitch_of_fingered_middle_c_01():

    clarinet = instrumenttools.Clarinet()

    assert clarinet.sounding_pitch_of_fingered_middle_c == 'bf'



from abjad import *


def test_TextSpanner___init___01():
    '''Init empty text spanner.
    '''

    spanner = spannertools.TextSpanner()
    assert isinstance(spanner, spannertools.TextSpanner)






from abjad.tools.pitchtools._IntervalSegment import _IntervalSegment


class MelodicDiatonicIntervalSegment(_IntervalSegment):
    '''.. versionadded:: 2.0

    Abjad model of melodic diatonic interval segment::

        abjad> pitchtools.MelodicDiatonicIntervalSegment('M2 M9 -m3 -P4')
        MelodicDiatonicIntervalSegment('+M2 +M9 -m3 -P4')

    Melodic diatonic interval segments are immutable.
    '''

    def __new__(self, arg):
        from abjad.tools import pitchtools
        if isinstance(arg, str):
            melodic_diatonic_interval_tokens = arg.split()
        else:
            melodic_diatonic_interval_tokens = arg
        mdis = []
        for token in melodic_diatonic_interval_tokens:
            mdi = pitchtools.MelodicDiatonicInterval(token)
            mdis.append(mdi)
        return tuple.__new__(self, mdis)

    ### OVERLOADS ###

    def __copy__(self):
        return type(self)(self.intervals)

    def __repr__(self):
        return "%s('%s')" % (type(self).__name__, ' '.join([str(x) for x in self]))

    ### PUBLIC ATTRIBUTES ###

    @property
    def harmonic_chromatic_interval_segment(self):
        from abjad.tools import pitchtools
        return pitchtools.HarmonicChromaticIntervalSegment(self.intervals)

    @property
    def harmonic_diatonic_interval_segment(self):
        from abjad.tools import pitchtools
        return pitchtools.HarmonicDiatonicIntervalSegment(self.intervals)

    @property
    def melodic_chromatic_interval_segment(self):
        from abjad.tools import pitchtools
        return pitchtools.MelodicChromaticIntervalSegment(self.intervals)




import copy
from collections import Iterable
from abjad import Fraction
from abjad.tools.durationtools import Duration
from abjad.tools.durationtools import Offset
from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools._IntervalNode import _IntervalNode
from abjad.tools.intervaltreetools._RedBlackTree import _RedBlackTree


class IntervalTree(_RedBlackTree):
    '''An augmented red-black tree for storing and searching for intervals of
    time (rather than pitch).

    This allows for the arbitrary placement of blocks of material along a
    time-line.  While this functionality could be achieved with Python's
    built-in collections, this class reduces the complexity of the search
    process, such as locating overlapping intervals.

    IntervalTrees can be instantiated without contents, or from a mixed
    collection of other IntervalTrees and / or BoundedIntervals.  The input
    will be parsed recursively ::

        abjad> from abjad.tools.intervaltreetools import IntervalTree
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> bi = BoundedInterval(0, 10)
        abjad> tree = IntervalTree([bi])

    '''

    __slots__ = ('_root', '_sentinel')

    def __init__(self, intervals = []):
        self._sentinel = _IntervalNode(0)
        self._sentinel.red = True
        self._sentinel.left = self._sentinel
        self._sentinel.right = self._sentinel
        self._sentinel.parent = self._sentinel
        self._root = self._sentinel
#        self._intervals = []
        self._insert(intervals)

    ### OVERLOADS ###

    def __contains__(self, item):
        if item in self._inorder:
            return True
        else:
            return False

    def __copy__(self):
        return IntervalTree([copy.copy(x) for x in self])

    def __eq__(self, other):
        if type(self) == type(other):
            if self[:] == other[:]:
                return True
        return False

    def __getitem__(self, item):
        return self._inorder.__getitem__(item)

    def __getslice__(self, start, end):
        return self._inorder.__getslice__(start, end)

    def __iter__(self):
        for interval in self._inorder:
            yield interval

    def __len__(self):
        def recurse(node):
            length = 0
            if node != self._sentinel:
                length += len(node.payload)
            if node.left != self._sentinel:
                length += recurse(node.left)
            if node.right != self._sentinel:
                length += recurse(node.right)
            return length
        if self._root != self._sentinel:
            return recurse(self._root)
        else:
            return 0

    def __repr__(self):
        if self:
            intervals = [repr(interval) for interval in self._inorder]
            return '%s([\n\t%s\n])' % (type(self).__name__, ',\n\t'.join(intervals))
        else:
            return '%s([])' % type(self).__name__

    # PRIVATE ATTRIBUTES

    @property
    def _inorder(self):
        if self:
            intervals = []
            nodes = tuple(self._sort_nodes_inorder())
            for node in nodes:
                intervals.extend(sorted(node.payload, key=lambda x: x.signature))
            return tuple(intervals)
        else:
            return ()

    ### PRIVATE METHODS ###

    def _insert(self, args):
        def recurse(x):
            if isinstance(x, BoundedInterval): # BoundedIntervals are Iterable!
                return [x]
            elif isinstance(x, Iterable) and \
            not isinstance(x, (basestring)):
                return [a for i in x for a in recurse(i)]

        intervals = recurse(args)
        assert all([isinstance(x, BoundedInterval) for x in intervals])

        for interval in intervals:
            node = self._find_by_key(interval.low)
            if node is not None:
                node.payload.append(interval)
            else:
                node = _IntervalNode(interval.low, interval)
                node.left = self._sentinel
                node.right = self._sentinel
                node.parent = self._sentinel
                self._insert_node(node)
#            self._intervals.append(interval)
        self._update_high_extrema()

    ### PUBLIC ATTRIBUTES ###

    @property
    def bounds(self):
        '''The lowest and highest values of the tree returned as a
        BoundedInterval.'''
        if self:
            return BoundedInterval(self.low, self.high)
        return None

    @property
    def high(self):
        '''The maximum high value of all intervals in the tree.
        Alias of high_max.'''
        return self.high_max

    @property
    def high_max(self):
        '''The maximum high value of all intervals in the tree.'''
        if self:
            return Offset(self._root.high_max)
        else:
            return None

    @property
    def high_min(self):
        '''The minimum high value of all intervals in the tree.'''
        if self:
            return Offset(self._root.high_min)
        else:
            return None

    @property
    def low(self):
        '''The minimum low value of all intervals in the tree.
        Alias of low_min.'''
        return self.low_min

    @property
    def low_max(self):
        '''The maximum low value of all intervals in the tree.'''
        if self:
            return Offset(self._find_maximum(self._root).key)
        else:
            return None

    @property
    def low_min(self):
        '''The minimum low value of all intervals in the tree.'''
        if self:
            return Offset(self._find_minimum(self._root).key)
        else:
            return None

    @property
    def magnitude(self):
        '''Absolute difference of the high and low values of the tree.'''
        if self:
            return Duration(self.high_max - self.low_min)
        else:
            return Duration(0)

    ### PRIVATE METHODS ###

    def _update_high_extrema(self):
        def recurse(node):
            max = min = node.payload[0].high
            for interval in node.payload[1:]:
                if max < interval.high:
                    max = interval.high
                if interval.high < min:
                    min = interval.high
            if node.left != self._sentinel:
                left_max, left_min = recurse(node.left)
                if max < left_max:
                    max = left_max
                if left_min < min:
                    min = left_min
            if node.right != self._sentinel:
                right_max, right_min = recurse(node.right)
                if max < right_max:
                    max = right_max
                if right_min < min:
                    min = right_min
            node.high_max = max
            node.high_min = min
            return max, min
        if self._root != self._sentinel:
            recurse(self._root)

### PUBLIC METHODS ###

    def find_intervals_intersecting_or_tangent_to_interval(self, *args):
        def recurse(node, low, high):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.key <= high and low <= node.high_max:
                for interval in node.payload:
                    if interval.low <= high and low <= interval.high:
                        intervals.append(interval)
            if node.left != self._sentinel \
                and self._find_minimum(node.left).key <= high \
                and low <= node.left.high_max:
                intervals.extend(recurse(node.left, low, high))
            if node.right != self._sentinel \
                and self._find_minimum(node.right).key <= high \
                and low <= node.right.high_max:
                intervals.extend(recurse(node.right, low, high))
            return intervals
        if len(args) == 1:
            assert isinstance(args[0], BoundedInterval)
            low, high = args[0].low, args[0].high
        elif len(args) == 2:
            low, high = args[0], args[1]
            assert all([isinstance(x, (int, Fraction)) for x in (low, high)])
            assert low <= high
        else:
            raise ValueError
        return tuple(sorted(recurse(self._root, low, high), key=lambda x: x.signature))

    def find_intervals_intersecting_or_tangent_to_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.key <= offset and offset <= node.high_max:
                for interval in node.payload:
                    if interval.low <= offset and offset <= interval.high:
                        intervals.append(interval)
            if node.left != self._sentinel and self._find_minimum(node.left).key <= offset \
                and offset <= node.left.high_max:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and self._find_minimum(node.right).key <= offset \
                and offset <= node.right.high_max:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_starting_after_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if offset < node.key:
                intervals.extend(node.payload)
            if node.left != self._sentinel and offset < self._find_maximum(node.left).key:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_starting_and_stopping_within_interval(self, *args):
        def recurse(node, low, high):
            intervals = []
            if node == self._sentinel:
                return intervals
            if low <= node.key and node.high_min <= high:
                for interval in node.payload:
                    if low <= interval.low and interval.high <= high:
                        intervals.append(interval)
            if node.left != self._sentinel \
                and low <= self._find_maximum(node.left).key \
                and node.left.high_min <= high:
                intervals.extend(recurse(node.left, low, high))
            if node.right != self._sentinel \
                and low <= self._find_maximum(node.right).key \
                and node.right.high_min <= high:
                intervals.extend(recurse(node.right, low, high))
            return intervals
        if len(args) == 1:
            assert isinstance(args[0], BoundedInterval)
            low, high = args[0].low, args[0].high
        elif len(args) == 2:
            low, high = args[0], args[1]
            assert all([isinstance(x, (int, Fraction)) for x in (low, high)])
            assert low <= high
        else:
            raise ValueError
        return tuple(sorted(recurse(self._root, low, high), key=lambda x: x.signature))

    def find_intervals_starting_at_offset(self, offset):
        assert isinstance(offset, (int, Fraction))
        node = self._find_by_key(offset)
        if node is not None and node != self._sentinel:
            return tuple(sorted(node.payload, key=lambda x: x.signature))
        else:
            return ()

    def find_intervals_starting_before_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.key < offset:
                intervals.extend(node.payload)
            if node.left != self._sentinel:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and self._find_minimum(node.right).key < offset:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_starting_or_stopping_at_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.key <= offset and offset <= node.high_max:
                for interval in node.payload:
                    if interval.low == offset or interval.high == offset:
                        intervals.append(interval)
            if node.left != self._sentinel and offset <= node.left.high_max:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and self._find_minimum(node.right).key <= offset \
                and offset <= node.right.high_max:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_starting_within_interval(self, *args):
        def recurse(node, low, high):
            intervals = []
            if node == self._sentinel:
                return intervals
            if low <= node.key <= high:
                intervals.extend(node.payload)
            if node.left != self._sentinel and \
                low <= self._find_maximum(node.left).key and \
                self._find_minimum(node.left).key <= high:
                intervals.extend(recurse(node.left, low, high))
            if node.right != self._sentinel and \
                low <= self._find_maximum(node.right).key and \
                self._find_minimum(node.right).key <= high:
                intervals.extend(recurse(node.right, low, high))
            return intervals
        if len(args) == 1:
            assert isinstance(args[0], BoundedInterval)
            low, high = args[0].low, args[0].high
        elif len(args) == 2:
            low, high = args[0], args[1]
            assert all([isinstance(x, (int, Fraction)) for x in (low, high)])
            assert low <= high
        else:
            raise ValueError
        return tuple(sorted(recurse(self._root, low, high), key=lambda x: x.signature))

    def find_intervals_stopping_after_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if offset < node.high_max:
                for interval in node.payload:
                    if offset < interval.high:
                        intervals.append(interval)
            if node.left != self._sentinel and offset < node.left.high_max:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and offset < node.right.high_max:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_stopping_at_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.high_min <= offset and offset <= node.high_max:
                for interval in node.payload:
                    if interval.high == offset:
                        intervals.append(interval)
            if node.left != self._sentinel and offset <= node.left.high_max:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and node.right.high_min <= offset:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_stopping_before_offset(self, offset):
        def recurse(node, offset):
            intervals = []
            if node == self._sentinel:
                return intervals
            if node.key <= offset and node.high_min < offset:
                for interval in node.payload:
                    if interval.high < offset:
                        intervals.append(interval)
            if node.left != self._sentinel and node.left.high_min < offset:
                intervals.extend(recurse(node.left, offset))
            if node.right != self._sentinel and node.right.high_min < offset:
                intervals.extend(recurse(node.right, offset))
            return intervals
        assert isinstance(offset, (int, Fraction))
        return tuple(sorted(recurse(self._root, offset), key=lambda x: x.signature))

    def find_intervals_stopping_within_interval(self, *args):
        def recurse(node, low, high):
            intervals = []
            if node == self._sentinel:
                return intervals
            if low <= node.high_max and node.high_min <= high:
                for interval in node.payload:
                    if low <= interval.high <= high:
                        intervals.append(interval)
            if node.left != self._sentinel and \
                low <= node.left.high_max and \
                node.left.high_min <= high:
                intervals.extend(recurse(node.left, low, high))
            if node.right != self._sentinel and \
                low <= node.right.high_max and \
                node.right.high_min <= high:
                intervals.extend(recurse(node.right, low, high))
            return intervals
        if len(args) == 1:
            assert isinstance(args[0], BoundedInterval)
            low, high = args[0].low, args[0].high
        elif len(args) == 2:
            low, high = args[0], args[1]
            assert all([isinstance(x, (int, Fraction)) for x in (low, high)])
            assert low <= high
        else:
            raise ValueError
        return tuple(sorted(recurse(self._root, low, high), key=lambda x: x.signature))



from abjad import *
from abjad.tools.pitchtools import NamedChromaticPitch


def test_NamedChromaticPitch___repr___01():
    '''Named chromatic pitch repr is evaluable.
    '''

    named_chromatic_pitch_1 = pitchtools.NamedChromaticPitch("cs''")
    named_chromatic_pitch_2 = eval(repr(named_chromatic_pitch_1))

    '''NamedChromaticPitch("cs''")'''

    assert isinstance(named_chromatic_pitch_1, pitchtools.NamedChromaticPitch)
    assert isinstance(named_chromatic_pitch_2, pitchtools.NamedChromaticPitch)




from abjad import *


def test_instrumenttools_Trombone_sounding_pitch_of_fingered_middle_c_01():

    trombone = instrumenttools.Trombone()

    assert trombone.sounding_pitch_of_fingered_middle_c == "c'"



def _get_documenting_directories():
   '''Get top-level Abjad directories to search for
   classes and functions to document in public API.
   '''

   return ['components', 'interfaces', 'marks', 'spanners', 'tools', ]



from abjad import *
import py.test


def test_MarkupCommand___init___01():
    '''`command` must be a non-empty string without spaces.'''
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand('', None, None)")
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand(3.14159, None, None)")
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand('one two', None, None)")


def test_MarkupCommand___init___02():
    '''`args` must be None or a non-zero length list.'''
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand('box', 3, None)")
    a = markuptools.MarkupCommand('box', None, None)
    b = markuptools.MarkupCommand('box', ['test'], None)


def test_MarkupCommand___init___03():
    '''`markup` must be None or a non-zero length list
        of strings and/or `MarkupCommand`s.'''
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand('box', None, 3)")
    assert py.test.raises(AssertionError, "markuptools.MarkupCommand('box', None, [3])")
    markuptools.MarkupCommand('box', None, None)
    markuptools.MarkupCommand('box', None, ['test'])
    markuptools.MarkupCommand('box', None, [markuptools.MarkupCommand('box', None, None)])


def test_MarkupCommand___init___04():
    '''`is_braced` defaults to `True`.'''
    a = markuptools.MarkupCommand('draw-circle', ['#1', '#0.1', '##f'], None)
    assert a.is_braced == True



from abjad.core import _Immutable
from abjad.core import _StrictComparator


class SchemeFunction(_StrictComparator, _Immutable):
    '''Abjad model of Scheme function::

        abjad> schemetools.SchemeFunction('magstep', -3)
        SchemeFunction('magstep', -3)

    Scheme functions are immutable.
    '''

    def __new__(klass, *args):
        self = object.__new__(klass)
        object.__setattr__(self, 'name', args[0])
        if 1 < len(args):
            object.__setattr__(self, 'args', args[1:])
        else:
            object.__setattr__(self, 'args', [])
        return self

    def __getnewargs__(self):
        newargs = [self.name]
        newargs.extend(self.args)
        return tuple(newargs)

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._repr_contents_string)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _repr_contents_string(self):
        result = []
        result.append('%r' % self.name)
        result.extend(self.args)
        result = ', '.join(str(x) for x in result)
        return result

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme function::

            abjad> scheme_function = schemetools.SchemeFunction('magstep', -3)
            abjad> scheme_function.format
            '#(magstep -3)'

        Return string.
        '''
        if len(self.args) == 0:
            body = self.name
        elif len(self.args) == 1:
            if isinstance(self.args[0], (int, float, long)):
                body = "(%s %s)" % (self.name, self.args[0])
            elif isinstance(self.args[0], str):
                body = "(%s '%s)" % (self.name, self.args[0])
            else:
                raise ValueError
        # TODO: Generalize for many arguments + parsing #
        else:
            raise NotImplementedError('multiple scheme arguments not yet implemented.')
        return '#' + body



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._BrassInstrument import _BrassInstrument
from abjad.tools.instrumenttools._WindInstrument import _WindInstrument


class FrenchHorn(_BrassInstrument, _WindInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the French horn::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.FrenchHorn()(staff)
        FrenchHorn('French Horn', 'Fr. hn.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { French Horn }
            \set Staff.shortInstrumentName = \markup { Fr. hn. }
            c'8
            d'8
            e'8
            f'8
        }

    The French horn targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'French Horn', short_instrument_name = 'Fr. hn.', target_context = None):
        _BrassInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('f')
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (-25, 17)



from abjad.tools.sequencetools._partition_sequence_elements_by_weights_exactly import _partition_sequence_elements_by_weights_exactly


def partition_sequence_cyclically_by_weights_exactly_with_overhang(sequence, weights):
    '''.. versionadded:: 1.1

    Partition `sequence` elements cyclically by `weights` exactly with overhang::

        abjad> from abjad.tools import sequencetools

    ::

        abjad> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
        abjad> sequencetools.partition_sequence_cyclically_by_weights_exactly_with_overhang(sequence, [12])
        [[3, 3, 3, 3], [4, 4, 4], [4, 5]]

    Return list of sequence element reference lists.

    .. versionchanged:: 2.0
        renamed ``sequencetools.group_sequence_elements_cyclically_by_weights_exactly_with_overhang()`` to
        ``sequencetools.partition_sequence_cyclically_by_weights_exactly_with_overhang()``.
    '''

    return _partition_sequence_elements_by_weights_exactly(
        sequence, weights, cyclic = True, overhang = True)



from abjad.core import _Immutable
from abjad.core import _StrictComparator


class SchemeBoolean(_Immutable):
    '''Abjad model of Scheme boolean::

        abjad> schemetools.SchemeBoolean(True)
        SchemeBoolean(True)

    Scheme variables are immutable.
    '''

    __slots__ = ('arg',)

    def __new__(klass, arg):
        self = object.__new__(klass)
        object.__setattr__(self, 'arg', bool(arg))
        return self

    def __getnewargs__(self):
        return (self.arg, )

    ### OVERLOADS ###

    def __eq__(self, arg):
        if isinstance(arg, SchemeBoolean):
            return arg.arg == self.arg
        return False

    def __repr__(self):
        return "%s(%r)" % (type(self).__name__, self.arg)

    def __str__(self):
        return self.format

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme boolean::

            abjad> scheme_boolean = schemetools.SchemeBoolean(True)
            abjad> scheme_boolean.format
            '##t'

        Return string.
        '''
        if self.arg:
            return '##t'
        return '##f'



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Clarinet import Clarinet


class EFlatClarinet(Clarinet):
    r'''.. versionadded:: 2.0

    Abjad model of the E-flat clarinet::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.EFlatClarinet()(staff)
        EFlatClarinet('Clarinet in E-flat', 'Cl. E-flat')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Clarinet in E-flat }
            \set Staff.shortInstrumentName = \markup { Cl. E-flat }
            c'8
            d'8
            e'8
            f'8
        }

    The E-flat clarinet targets staff context by default.
    '''

    def __init__(self, instrument_name = 'Clarinet in E-flat',
        short_instrument_name = 'Cl. E-flat', target_context = None):
        Clarinet.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch("ef'")
        self.primary_clefs = [contexttools.ClefMark('treble')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (-7, 36)



from abjad.core import _Immutable


class OmissionIndicator(_Immutable):
    '''.. versionadded:: 2.0

    Indicator of missing chord tones.

    Value object that can not be chnaged after instantiation.
    '''

    def __init__(self):
        pass



from abjad.tools.intervaltreetools import BoundedInterval
from abjad.tools.intervaltreetools import IntervalTree
from abjad.tools.intervaltreetools import resolve_overlaps_between_nonoverlapping_trees


def test_intervaltreetools_resolve_overlaps_between_nonoverlapping_trees_01():
    a = IntervalTree(BoundedInterval(0, 4, {'a': 1}))
    b = IntervalTree(BoundedInterval(1, 5, {'b': 2}))
    c = IntervalTree(BoundedInterval(2, 6, {'c': 3}))
    d = IntervalTree(BoundedInterval(1, 3, {'d': 4}))
    result = resolve_overlaps_between_nonoverlapping_trees([a, b, c, d])
    assert result == \
    IntervalTree([
        BoundedInterval(0, 4, {'a': 1}),
        BoundedInterval(4, 5, {'b': 2}),
        BoundedInterval(5, 6, {'c': 3})
    ])



from abjad import *


def test_SchemeFunction___init___01():
    '''Scheme function with only a name and no arguments.'''

    t = Staff("c'8 d'8 e'8 f'8")
    t.override.time_signature.break_visibility = schemetools.SchemeFunction('end-of-line-invisible')

    r'''
    \new Staff \with {
        \override TimeSignature #'break-visibility = #end-of-line-invisible
    } {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert t.format == "\\new Staff \\with {\n\t\\override TimeSignature #'break-visibility = #end-of-line-invisible\n} {\n\tc'8\n\td'8\n\te'8\n\tf'8\n}"


def test_SchemeFunction___init___02():
    '''Scheme function with a name and a single numeric argument.'''

    staff = Staff("c'8 d'8 e'8 f'8")
    staff.override.staff_symbol.staff_space = schemetools.SchemeFunction('magstep', -3)
    staff.override.staff_symbol.thickness = schemetools.SchemeFunction('magstep', -3)

    r'''
    \new Staff \with {
        \override StaffSymbol #'staff-space = #(magstep -3)
        \override StaffSymbol #'thickness = #(magstep -3)
    } {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert componenttools.is_well_formed_component(staff)
    assert staff.format == "\\new Staff \\with {\n\t\\override StaffSymbol #'staff-space = #(magstep -3)\n\t\\override StaffSymbol #'thickness = #(magstep -3)\n} {\n\tc'8\n\td'8\n\te'8\n\tf'8\n}"



from abjad.core import _Immutable


class MarkupCommand(_Immutable):
    r'''Abjad model of a LilyPond markup command::

        abjad> circle = markuptools.MarkupCommand('draw-circle', ['#2.5', '#0.1', '##f'], None)
        abjad> square = markuptools.MarkupCommand('rounded-box', None, ['hello?'])
        abjad> line = markuptools.MarkupCommand('line', None, [square, 'wow!'])
        abjad> rotate = markuptools.MarkupCommand('rotate', ['#60'], [line])
        abjad> combine = markuptools.MarkupCommand('combine', None, [rotate, circle], is_braced = False)

    ::

        abjad> print combine
        \combine \rotate #60 \line { \rounded-box hello? wow! } \draw-circle #2.5 #0.1 ##f

    Insert markup command in markup to attach to score components::

        abjad> note = Note("c'4")

    ::

        abjad> markup = markuptools.Markup(combine)

    ::

        abjad> markup(note)
        Markup('\\combine \\rotate #60 \\line { \\rounded-box hello? wow! } \\draw-circle #2.5 #0.1 ##f')

    ::

        abjad> f(note)
        c'4 \markup { \combine \rotate #60 \line { \rounded-box hello? wow! } \draw-circle #2.5 #0.1 ##f }

    Markup commands are immutable.
    '''

    # TODO: Implement a multi-line, indented version for human readability. #

    __slots__ = ('_args', '_is_braced', '_command', '_markup')

    def __init__(self, command, args, markup, is_braced = True):
        assert isinstance(command, str) \
            and len(command) and command.find(' ') == -1
        assert isinstance(args, type(None)) or \
            (isinstance(args, (list, tuple)) and len(args))
        assert isinstance(markup, type(None)) or \
            (isinstance(markup, (list, tuple)) and len(markup))
        if markup:
            assert all([isinstance(x, (MarkupCommand, str)) for x in markup])

        object.__setattr__(self, '_is_braced', bool(is_braced))
        object.__setattr__(self, '_command', command)

        if args:
            object.__setattr__(self, '_args', tuple(args))
        else:
            object.__setattr__(self, '_args', args)

        if markup:
            object.__setattr__(self, '_markup', tuple(markup))
        else:
            object.__setattr__(self, '_markup', markup)

    # OVERRIDES #

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.args == arg.args and \
                self.is_braced == arg.is_braced and \
                self.command == arg.command and \
                self.markup == arg.markup:
                return True
        return False

    def __repr__(self):
        return '%s(%r, %s, %s)' % (type(self).__name__, self.command, self.args, self.markup)

    def __str__(self):
        return self.format

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_pieces(self):
        indent_delimiter = ''
        parts = [r'\%s' % self.command]
        if self.args is not None:
            for arg in self.args:
                if 'format' in dir(arg) and not isinstance(arg, str):
                    parts[0] += ' %s' % arg.format
                else:
                    parts[0] += ' %s' % arg
        if self.markup is not None:
            for markup in self.markup:
                if '_format_pieces' in dir(markup) and not isinstance(markup, str):
                    parts.extend([indent_delimiter + x for x in markup._format_pieces])
                else: # markup is a string
                    parts.append(indent_delimiter + self._escape_string(markup))
        if self.is_braced and self.markup and 1 < len(self.markup):
            parts[0] += ' {'
            parts.append('}')
        return parts

    @property
    def _report_pieces(self):
        indent_delimiter = '\t'
        parts = [r'\%s' % self.command]
        if self.args is not None:
            for arg in self.args:
                if 'format' in dir(arg) and not isinstance(arg, str):
                    parts[0] += ' %s' % arg.format
                else:
                    parts[0] += ' %s' % arg
        if self.markup is not None:
            for markup in self.markup:
                if '_report_pieces' in dir(markup) and not isinstance(markup, str):
                    parts.extend([indent_delimiter + x for x in markup._report_pieces])
                else: # markup is a string
                    parts.append(indent_delimiter + self._escape_string(markup))
        if self.is_braced and self.markup and 1 < len(self.markup):
            parts[0] += ' {'
            parts.append('}')
        return parts

    ### PRIVATE METHODS ###

    def _escape_string(self, string):
        if -1 == string.find(' '):
            return string

        string = repr(string)
        if string.startswith("'") and string.endswith("'"):
            string = string.replace('"', '\"')
            string = '"' + string[1:]
            string = string[:-1] + '"'
        return string

    ### PUBLIC ATTRIBUTES ###

    @property
    def args(self):
        r'''Read-only tuple of markup command arguments.'''
        return self._args

    @property
    def command(self):
        r'''Read-only string of markup command command-name.'''
        return self._command

    @property
    def format(self):
        r'''Read-only format of markup command::

            abjad> markup_command = markuptools.MarkupCommand('draw-circle', ['#2.5', '#0.1', '##f'], None)
            abjad> markup_command.format
            '\\draw-circle #2.5 #0.1 ##f'

        Return list of strings.
        '''

        return ' '.join(self._format_pieces)

    @property
    def is_braced(self):
        r'''Read-only boolean of markup command bracing.'''
        return self._is_braced

    @property
    def markup(self):
        r'''Read-only tuple of markup command's child markup.'''
        return self._markup

    ### PUBLIC METHODS ###

    def report(self, output = 'screen'):
        '''Report, in an indented human-readable format, the structure of a formatted MarkupCommand.'''
        if output == 'screen':
            print '\n'.join(self._report_pieces)
        else:
            return '\n'.join(self._report_pieces)



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Bassoon import Bassoon


class Contrabassoon(Bassoon):
    r'''.. versionadded:: 2.0

    Abjad model of the contrabassoon::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")
        abjad> contexttools.ClefMark('bass')(staff)
        ClefMark('bass')(Staff{4})

    ::

        abjad> instrumenttools.Contrabassoon()(staff)
        Contrabassoon('Contrabassoon', 'Contrabsn.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \clef "bass"
            \set Staff.instrumentName = \markup { Contrabassoon }
            \set Staff.shortInstrumentName = \markup { Contrabsn. }
            c'8
            d'8
            e'8
            f'8
        }

    The contrabassoon targets staff context by default.
    '''

    def __init__(self, instrument_name = 'Contrabassoon',
        short_instrument_name = 'Contrabsn.', target_context = None):
        Bassoon.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('c')
        self.primary_clefs = [contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (-38, -2)



from abjad.tools.contexttools._Context import _Context


class Voice(_Context):
    r'''Abjad model of a voice:

    ::

        abjad> voice = Voice("c'8 d'8 e'8 f'8")
        abjad> f(voice)
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }

    Return voice object.
    '''

    __slots__ = ()

    def __init__(self, music = None, **kwargs):
        _Context.__init__(self, music)
        self.context = 'Voice'
        self._initialize_keyword_values(**kwargs)



import os


def _get_public_names_in_module(module_file):
    '''Collects and returns all functions defined in module_file.'''

    result = []
    module_file = module_file.replace(os.sep, '.')
    mod = __import__(module_file, fromlist=['*'])
    for key, value in vars(mod).items():
        # if not a private function
        if not key.startswith('_'):
            if getattr(value, '__module__', None) == module_file:
                result.append(value)
    return result



from abjad.tools.lilyfiletools._BlockAttributed import _BlockAttributed


class LayoutBlock(_BlockAttributed):
    r'''.. versionadded:: 2.0

    Abjad model of LilyPond input file layout block.
    '''

    def __init__(self):
        _BlockAttributed.__init__(self)
        self._escaped_name = r'\layout'
        self._contexts = []

    # PRIVATE ATTRIUBTES #

    @property
    def _formatted_context_specifications(self):
        result = []
        for context_specification in self.contexts:
            result.append(r'\context {')
            for x in context_specification:
                result.append('\t' + str(x))
            result.append('}')
        return result

    ### PUBLIC ATTRIBUTES ###

    @property
    def contexts(self):
        return self._contexts



from abjad.tools.containertools.Container._ContainerFormatter import _ContainerFormatter
from abjad.tools.componenttools._Component import _Component
import copy


class Container(_Component):
    '''Abjad model of a music container:

    ::

        abjad> container = Container("c'8 d'8 e'8 f'8")
        abjad> f(container)
        {
            c'8
            d'8
            e'8
            f'8
        }

    Return container object.
    '''

    __slots__ = ('_formatter', '_music', '_parallel', )

    def __init__(self, music = None, **kwargs):
        _Component.__init__(self)
        self._initialize_music(music)
        self._formatter = _ContainerFormatter(self)
        self._parallel = False
        self._initialize_keyword_values(**kwargs)

    ### OVERLOADS ###

    def __add__(self, expr):
        '''Concatenate containers self and expr.
        The operation c = a + b returns a new Container c with
        the content of both a and b.
        The operation is non-commutative: the content of the first
        operand will be placed before the content of the second operand.
        '''
        from abjad.tools import componenttools
        from abjad.tools import containertools
        left = componenttools.copy_components_and_fracture_crossing_spanners([self])[0]
        right = componenttools.copy_components_and_fracture_crossing_spanners([expr])[0]
        return containertools.fuse_like_named_contiguous_containers_in_expr([left, right])

    def __contains__(self, expr):
        '''True if expr is in container, otherwise False.'''
        for x in self._music:
            if x is expr:
                return True
        else:
            return False

    def __copy__(self, *args):
        new = _Component.__copy__(self, *args)
        new.is_parallel = self.is_parallel
        return new

    def __deepcopy__(self, memo):
        new = self.__copy__()
        for component in self.music:
            new_component = copy.deepcopy(component)
            new.append(new_component)
        return new

    def __delitem__(self, i):
        '''Find component(s) at index or slice 'i' in container.
        Detach component(s) from parentage.
        Withdraw component(s) from crossing spanners.
        Preserve spanners that component(s) cover(s).
        '''
        from abjad.tools.componenttools._switch_components_to_parent import _switch_components_to_parent
        from abjad.tools.spannertools._withdraw_components_in_expr_from_crossing_spanners import _withdraw_components_in_expr_from_crossing_spanners
        components = self[i]
        if not isinstance(components, list):
            components = [components]
        _withdraw_components_in_expr_from_crossing_spanners(components)
        _switch_components_to_parent(components, None)

    def __getitem__(self, i):
        '''Return component at index i in container.
        Shallow traversal of container for numeric indices only.
        '''
        return self._music[i]

    def __iadd__(self, expr):
        '''__iadd__ avoids unnecessary copying of structures.
        '''
        from abjad.tools import componenttools
        from abjad.tools import containertools
        return containertools.fuse_like_named_contiguous_containers_in_expr([self,
            componenttools.copy_components_and_fracture_crossing_spanners([expr])[0]])

    def __imul__(self, total):
        '''Multiply contents of container 'total' times.
        Return multiplied container.
        '''
        from abjad.tools import containertools
        return containertools.repeat_contents_of_container(self, total = total)

    def __len__(self):
        '''Return nonnegative integer number of components in container.
        '''
        return len(self._music)

    def __radd__(self, expr):
        '''Extend container by contents of expr to the right.
        '''
        return self + expr

    def __repr__(self):
        '''String format of container for interpreter display.
        '''
        return self._compact_representation

    def __setitem__(self, i, expr):
        '''Set 'expr' in self at nonnegative integer index i.
        Or, set 'expr' in self at slice i.
        Find spanners that dominate self[i] and children of self[i].
        Replace contents at self[i] with 'expr'.
        Reattach spanners to new contents.
        This operation leaves all score trees always in tact.
        '''
        from abjad.tools import componenttools
        from abjad.tools import iotools
        from abjad.tools import spannertools
        from abjad.tools.spannertools._withdraw_components_in_expr_from_crossing_spanners import _withdraw_components_in_expr_from_crossing_spanners
        # item assignment
        if isinstance(i, int):
            assert componenttools.all_are_components([expr])
            old = self[i]
            spanners_receipt = spannertools.get_spanners_that_dominate_components([old])
            # must withdraw from spanners before parentage!
            # otherwise begin / end assessments don't work!
            _withdraw_components_in_expr_from_crossing_spanners([expr])
            expr._parentage._switch(self)
            self._music.insert(i, expr)
            componenttools.remove_component_subtree_from_score_and_spanners([old])
            for spanner, index in spanners_receipt:
                spanner._insert(index, expr)
                expr._spanners.add(spanner)
        # slice assignment
        else:
            if isinstance(expr, str):
                container = iotools.parse_lilypond_input_string(expr)
                expr = container[:]
            assert componenttools.all_are_components(expr)
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            spanners_receipt = spannertools.get_spanners_that_dominate_container_components_from_to(
                self, start, stop)
            componenttools.remove_component_subtree_from_score_and_spanners(old)
            # must withdraw before setting in self!
            # otherwise circular withdraw ensues!
            _withdraw_components_in_expr_from_crossing_spanners(expr)
            self._music[start:start] = expr
            for component in expr:
                component._parentage._switch(self)
            for spanner, index in spanners_receipt:
                for component in reversed(expr):
                    spanner._insert(index, component)
                    component._spanners.add(spanner)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _compact_representation(self):
        '''Compact form used in spanner display.
        '''
        if not self.is_parallel:
            return '{%s}' % self._summary
        else:
            return '<<%s>>' % self._summary

    @property
    def _summary(self):
        '''Formatted summary of container contents for repr output.
        '''
        if 0 < len(self):
            return ', '.join([str(x) for x in self._music])
        else:
            #return ' '
            return ''

    ### PUBLIC ATTRIBUTES ###

    @property
    def contents_duration(self):
        from abjad.tools import durationtools
        if self.is_parallel:
            return max([durationtools.Duration(0)] + [x.preprolated_duration for x in self])
        else:
            duration = durationtools.Duration(0)
            for x in self:
                duration += x.preprolated_duration
            return duration

    @property
    def duration_in_seconds(self):
        from abjad.tools import durationtools
        if self.is_parallel:
            return max([durationtools.Duration(0)] + [x.duration_in_seconds for x in self])
        else:
            duration = durationtools.Duration(0)
            for leaf in self.leaves:
                duration += leaf.duration_in_seconds
            return duration

    @apply
    def is_parallel():
        def fget(self):
            r'''Get parallel container::

                abjad> container = Container([Voice("c'8 d'8 e'8"), Voice('g4.')])

            ::

                abjad> f(container)
                {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                }

            ::

                abjad> container.is_parallel
                False

            Return boolean.

            Set parallel container::

                abjad> container.is_parallel = True

            ::

                abjad> f(container)
                <<
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                >>

            Return none.
            '''
            return self._parallel
        def fset(self, expr):
            from abjad.tools.contexttools._Context import _Context
            from abjad.tools import componenttools
            #assert isinstance(expr, (bool, type(None)))
            assert isinstance(expr, type(True))
            if expr == True:
                assert componenttools.all_are_components(self._music, klasses = (_Context, ))
            self._parallel = expr
            self._mark_entire_score_tree_for_later_update('prolated')
        return property(**locals())

    @property
    def leaves(self):
        '''Read-only tuple of leaves in container::

            abjad> container = Container("c'8 d'8 e'8")

        ::

            abjad> container.leaves
            (Note("c'8"), Note("d'8"), Note("e'8"))

        Return tuple of zero or more leaves.
        '''
        from abjad.tools import leaftools
        return tuple(leaftools.iterate_leaves_forward_in_expr(self))

    @property
    def music(self):
        '''Read-only tuple of components in container::

            abjad> container = Container("c'8 d'8 e'8")

        ::

            abjad> container.music
            (Note("c'8"), Note("d'8"), Note("e'8"))

        Return tuple or zero or more components.
        '''
        return tuple(self._music)

    @property
    def preprolated_duration(self):
        return self.contents_duration

    ### PRIVATE METHODS ###

    def _initialize_music(self, music):
        from abjad.tools import componenttools
        from abjad.tools import iotools
        from abjad.tools.componenttools._switch_components_to_parent import _switch_components_to_parent
        music = music or []
        if componenttools.all_are_contiguous_components_in_same_thread(music):
            parent, index, stop_index = componenttools.get_parent_and_start_stop_indices_of_components(
                music)
            self._music = list(music)
            _switch_components_to_parent(self._music, self)
            if parent is not None:
                parent._music.insert(index, self)
                self._parentage._switch(parent)
        elif isinstance(music, str):
            music_container = iotools.parse_lilypond_input_string(music)
            self._music = []
            self[:] = music_container.music
        else:
            raise TypeError('can not initialize container from "%s".' % str(music))

    def _is_one_of_my_first_leaves(self, leaf):
        return leaf in self._navigator._contemporaneous_start_contents

    def _is_one_of_my_last_leaves(self, leaf):
        return leaf in self._navigator._contemporaneous_stop_contents

    ### PUBLIC METHODS ###

    def append(self, component):
        r'''Append `component` to container::

            abjad> container = Container("c'8 d'8 e'8")
            abjad> beam = spannertools.BeamSpanner(container.music)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
            }

        ::

            abjad> container.append(Note("f'8"))

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
                f'8
            }

        Return none.
        '''
        self[len(self):len(self)] = [component]

    def extend(self, expr):
        '''Extend `expr` against container::

            abjad> container = Container("c'8 d'8 e'8")
            abjad> beam = spannertools.BeamSpanner(container.music)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
            }

        ::

            abjad> container.extend([Note("cs'8"), Note("ds'8"), Note("es'8")])

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
                cs'8
                ds'8
                es'8
            }

        Return none.

        .. versionadded:: 2.3
            ``expr`` may now be a LilyPond input string.
        '''
        self[len(self):len(self)] = expr[:]
        #return self

    def index(self, component):
        '''Index `component` in container::

            abjad> container = Container("c'8 d'8 e'8")

        ::

            abjad> note = container[-1]
            abjad> note
            Note("e'8")

        ::

            abjad> container.index(note)
            2

        Return nonnegative integer.
        '''
        for i, element in enumerate(self._music):
            if element is component:
                return i
        else:
            raise ValueError('component "%s" not in Abjad container.' % component)

    def insert(self, i, component):
        '''Insert `component` in container at index `i`::

            abjad> container = Container("c'8 d'8 e'8")
            abjad> beam = spannertools.BeamSpanner(container.music)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
            }

        ::

            abjad> container.insert(1, Note("cs'8"))

        ::

            abjad> f(container)
            {
                c'8 [
                cs'8
                d'8
                e'8 ]
            }

        Return none.
        '''
        self[i:i] = [component]

    def pop(self, i = -1):
        '''Pop component at index `i` from container::

            abjad> container = Container("c'8 d'8 e'8")
            abjad> beam = spannertools.BeamSpanner(container.music)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
            }

        ::

            abjad> container.pop(-1)
            Note("e'8")

        ::

            abjad> f(container)
            {
                c'8 [
                d'8 ]
            }

        Return component.
        '''
        component = self[i]
        del(self[i])
        return component

    def remove(self, component):
        '''Remove `component` from container::

            abjad> container = Container("c'8 d'8 e'8")
            abjad> beam = spannertools.BeamSpanner(container.music)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8
                e'8 ]
            }

        ::

            abjad> note = container[-1]
            abjad> note
            Note("e'8")

        ::

            abjad> container.remove(note)

        ::

            abjad> f(container)
            {
                c'8 [
                d'8 ]
            }

        Return none.
        '''
        i = self.index(component)
        del(self[i])
        #return component



from abjad.tools.intervaltreetools import IntervalTree
from abjad.tools.intervaltreetools import calculate_min_mean_and_max_depth_of_intervals
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction


def test_intervaltreetools_calculate_min_mean_and_max_depth_of_intervals_01():
    tree = IntervalTree([])
    result = calculate_min_mean_and_max_depth_of_intervals(tree)
    assert result is None


def test_intervaltreetools_calculate_min_mean_and_max_depth_of_intervals_02():
    tree = IntervalTree(_make_test_intervals())
    result = calculate_min_mean_and_max_depth_of_intervals(tree)
    assert result == (0, Fraction(45, 37), 3)



from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_partition_sequence_once_by_weights_exactly_without_overhang_01():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_once_by_weights_exactly_without_overhang(
        sequence, [3, 9])
    assert groups == [[3], [3, 3, 3]]



def fuse_leaves_in_container_once_by_counts_into_little_endian_rests(container, counts):
    '''.. versionadded:: 1.1

    Fuse leaves in `container` once by `counts` into little-endian rests.
    '''
    from abjad.tools.resttools.Rest import Rest
    from abjad.tools.leaftools._fuse_leaves_in_container_once_by_counts \
        import _fuse_leaves_in_container_once_by_counts

    return _fuse_leaves_in_container_once_by_counts(container, counts,
        target_type = Rest, direction = 'little-endian')





from abjad.core import _Immutable
from abjad.core import _StrictComparator


class SchemeVariable(_StrictComparator, _Immutable):
    '''Abjad model of Scheme variable::

        abjad> schemetools.SchemeVariable('grace')
        SchemeVariable('grace')

    Scheme variables are immutable.
    '''

    #def __init__(self, string):
    #    self._string = string
    def __new__(klass, string):
        self = object.__new__(klass)
        object.__setattr__(self, '_string', string)
        return self

    def __getnewargs__(self):
        return (self._string, )

    ### OVERLOADS ###

    def __repr__(self):
        return "%s(%r)" % (type(self).__name__, self._string)

    def __str__(self):
        return self.format

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme variable:

            abjad> scheme_variable = schemetools.SchemeVariable('UP')
            abjad> scheme_variable.format
            '#UP'

        Return string.
        '''
        return '#%s' % self._string



from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad import Fraction


def scale_aggregate_magnitude_by_rational(intervals, rational):
    '''Scale the aggregate magnitude of all intervals in `intervals` by
    `rational`, maintaining the original low offset ::

        abjad> from abjad.tools import intervaltreetools
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> from abjad.tools.intervaltreetools import IntervalTree

    ::

        abjad> a = BoundedInterval(-1, 3)
        abjad> b = BoundedInterval(6, 12)
        abjad> c = BoundedInterval(9, 16)
        abjad> tree = IntervalTree([a, b, c])
        abjad> intervaltreetools.scale_aggregate_magnitude_by_rational(tree, Fraction(1, 3))
        IntervalTree([
            BoundedInterval(Offset(-1, 1), Offset(1, 3), {}),
            BoundedInterval(Offset(4, 3), Offset(10, 3), {}),
            BoundedInterval(Offset(7, 3), Offset(14, 3), {})
        ])

    Return interval tree.
    '''

    assert isinstance(rational, (int, Fraction)) and 0 < rational
    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree or rational == 1:
        return tree

    return IntervalTree([
        x.shift_to_rational(
            ((x.low - tree.low) * rational) + tree.low).scale_by_rational(rational)\
            for x in tree
    ])



from abjad.tools.intervaltreetools import *


def test_IntervalTree___len____01():
    '''IntervalTree containing no BoundedIntervals report 0 length.'''
    tree = IntervalTree([])
    assert len(tree) == 0


def test_IntervalTree___len____02():
    '''IntervalTree reports total count of BoundedIntervals contained.'''
    block_a = BoundedInterval(0, 5, {'a': 1})
    block_b = BoundedInterval(0, 10, {'b': 2})
    block_c = BoundedInterval(5, 20, {'c': 3})
    tree = IntervalTree([block_a, block_b, block_c])
    assert len(tree) == 3



from abjad.tools.pitchtools._IntervalSet import _IntervalSet


class HarmonicDiatonicIntervalSet(_IntervalSet):
    '''.. versionadded:: 2.0

    Abjad model of harmonic diatonic interval set::

        abjad> pitchtools.HarmonicDiatonicIntervalSet('m2 m2 M2 M9')
        HarmonicDiatonicIntervalSet('m2 M2 M9')

    Harmonic diatonic interval sets are immutable.
    '''

    def __new__(self, arg):
        from abjad.tools import pitchtools
        if isinstance(arg, str):
            interval_tokens = arg.split()
        else:
            interval_tokens = arg
        hdis = [pitchtools.HarmonicDiatonicInterval(x) for x in interval_tokens]
        return frozenset.__new__(self, hdis)

    ### OVERLOADS ###

    def __copy__(self):
        return type(self)(self)

    def __repr__(self):
        return "%s('%s')" % (type(self).__name__, self._format_string)

    def __str__(self):
        return '{%s}' % self._format_string

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        return ' '.join([str(x) for x in sorted(self.harmonic_diatonic_intervals)])

    ### PUBLIC ATTRIBUTES ###

    @property
    def harmonic_chromatic_interval_set(self):
        from abjad.tools import pitchtools
        return pitchtools.HarmonicChromaticIntervalSet(self)

    @property
    #def intervals(self):
    def harmonic_diatonic_intervals(self):
        return set(self)

    @property
    #def numbers(self):
    def harmonic_diatonic_interval_numbers(self):
        return set([interval.number for interval in self])




from abjad.tools.pitchtools._Numbered import _Numbered
from abjad.tools.pitchtools._PitchClass import _PitchClass


#class _NumberedPitchClass(_PitchClass, _Numbered, _FlexEqualityComparator):
class _NumberedPitchClass(_PitchClass, _Numbered):
    '''.. versionadded:: 2.0

    Numbered pitch-class base class.
    '''

    __slots__ = ()




from abjad import *


def test_MelodicDiatonicInterval___eq___01():

    diatonic_interval_1 = pitchtools.MelodicDiatonicInterval('minor', 2)
    diatonic_interval_2 = pitchtools.MelodicDiatonicInterval('minor', 2)

    assert diatonic_interval_1 == diatonic_interval_2



def test_MelodicDiatonicInterval___eq___02():

    diatonic_interval_1 = pitchtools.MelodicDiatonicInterval('minor', 2)
    diatonic_interval_2 = pitchtools.MelodicDiatonicInterval('augmented', 1)

    assert not diatonic_interval_1 == diatonic_interval_2




from abjad.tools.spannertools.PhrasingSlurSpanner._PhrasingSlurSpannerFormatInterface import _PhrasingSlurSpannerFormatInterface
from abjad.tools.spannertools.Spanner import Spanner


class PhrasingSlurSpanner(Spanner):
    r'''Abjad phrasing slur spanner::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> spannertools.PhrasingSlurSpanner(staff[:])
        PhrasingSlurSpanner(c'8, d'8, e'8, f'8)

    ::

        abjad> f(staff)
        \new Staff {
            c'8 \(
            d'8
            e'8
            f'8 \)
        }

    Return phrasing slur spanner.
    '''

    def __init__(self, components = None):
        Spanner.__init__(self, components)
        self._format = _PhrasingSlurSpannerFormatInterface(self)






from abjad import Fraction
from abjad.core import _Immutable
from abjad.tools.durationtools import Duration
from abjad.tools.durationtools import Offset

class BoundedInterval(dict, _Immutable):
    '''A low / high pair, carrying some metadata.'''

    __slots__ = ('_high', '_low', )

    def __init__(self, *args):
        dict.__init__(self)
        if len(args) == 1 and isinstance(args[0], BoundedInterval):
            low, high, data = args[0].low, args[0].high, args[0]
        elif len(args) == 2:
            low, high, data = args[0], args[1], {}
        elif len(args) == 3:
            low, high, data = args[0], args[1], args[2]
        assert isinstance(low, (int, Fraction))
        assert isinstance(high, (int, Fraction))
        assert low < high
        assert isinstance(data, dict)
        object.__setattr__(self, '_low', Offset(low))
        object.__setattr__(self, '_high', Offset(high))
        self.update(data)

    ### OVERLOADS ###

    def __eq__(self, other):
        if type(other) == type(self):
            if other.low == self.low:
                if other.high == self.high:
                    if dict.__eq__(self, other):
                        return True
        return False

    def __hash__(self):
        return id(self)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return '%s(%s, %s, %s)' % \
            (type(self).__name__, \
            repr(self.low), \
            repr(self.high), \
            dict.__repr__(self))

    ### PUBLIC ATTRIBUTES ###

    @property
    def centroid(self):
        '''Center point of low and high bounds.'''
        return Offset(self.high + self.low) / 2

    @property
    def high(self):
        '''High bound.'''
        return self._high

    @property
    def low(self):
        '''Low bound.'''
        return self._low

    @property
    def magnitude(self):
        '''High bound minus low bound.'''
        return Duration(self.high - self.low)

    @property
    def signature(self):
        '''Tuple of low bound and high bound.'''
        return (self.low, self.high)

    ### PUBLIC METHODS ###

    def get_overlap_with_interval(self, interval):
        '''Return amount of overlap with `interval`.'''
        assert isinstance(interval, BoundedInterval)
        if not self.is_overlapped_by_interval(interval):
            return 0
        elif self.is_container_of_interval(interval):
            return interval.magnitude
        elif self.is_contained_by_interval(interval):
            return self.magnitude
        elif self.low < interval.low:
            return self.high - interval.low
        else:
            return interval.high - self.low

    def is_container_of_interval(self, interval):
        '''True if interval contains `interval`.'''
        assert isinstance(interval, BoundedInterval)
        if self.low <= interval.low and interval.high <= self.high:
            return True
        else:
            return False

    def is_contained_by_interval(self, interval):
        '''True if interval is contained by `interval`.'''
        assert isinstance(interval, BoundedInterval)
        if interval.low <= self.low and self.high <= interval.high:
            return True
        else:
            return False

    def is_overlapped_by_interval(self, interval):
        '''True if interval is overlapped by `interval`.'''
        assert isinstance(interval, BoundedInterval)
        if self.is_container_of_interval(interval):
            return True
        elif self.is_contained_by_interval(interval):
            return True
        elif self.low < interval.low < self.high:
            return True
        elif self.low == interval.low:
            return True
        elif self.high == interval.high:
            return True
        elif self.low < interval.high < self.high:
            return True
        else:
            return False

    def is_tangent_to_interval(self, interval):
        '''True if interval is tangent to `interval`.'''
        assert isinstance(interval, BoundedInterval)
        if self.high == interval.low or interval.high == self.low:
            return True
        else:
            return False

    def scale_by_rational(self, rational):
        assert isinstance(rational, (int, Fraction))
        assert 0 <= rational
        if rational != 1:
            new_magnitude = (self.high - self.low) * rational
            return type(self)(BoundedInterval(self.low, self.low + new_magnitude, self))
        else:
            return self

    def scale_to_rational(self, rational):
        assert isinstance(rational, (int, Fraction))
        assert 0 <= rational
        if rational != self.magnitude:
            return type(self)(BoundedInterval(self.low, self.low + rational, self))
        else:
            return self

    def shift_by_rational(self, rational):
        assert isinstance(rational, (int, Fraction))
        if rational != 0:
            return type(self)(BoundedInterval(self.low + rational, self.high + rational, self))
        else:
            return self

    def shift_to_rational(self, rational):
        assert isinstance(rational, (int, Fraction))
        if rational != self.low:
            magnitude = self.high - self.low
            return type(self)(BoundedInterval(rational, rational + magnitude, self))
        else:
            return self

    def split_at_rational(self, rational):
        assert isinstance(rational, (int, Fraction))
        if self.low < rational < self.high:
            return (type(self)(BoundedInterval(self.low, rational, self)),
                    type(self)(BoundedInterval(rational, self.high, self)))
        else:
            return (self,)



from abjad.tools.scoretools.StaffGroup.StaffGroup import StaffGroup


class PianoStaff(StaffGroup):
    r'''Abjad model of piano staff::

        abjad> staff_1 = Staff("c'4 d'4 e'4 f'4 g'1")
        abjad> staff_2 = Staff("g2 f2 e1")

    ::

        abjad> piano_staff = scoretools.PianoStaff([staff_1, staff_2])

    ::

        abjad> f(piano_staff)
        \new PianoStaff <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
                g'1
            }
            \new Staff {
                g2
                f2
                e1
            }
        >>

    Return piano staff.
    '''

    def __init__(self, music):
        StaffGroup.__init__(self, music)
        self.context = 'PianoStaff'





from abjad import *


def test_SchemeNumber___init___01():

    t = Staff([])
    t.override.staff_symbol.thickness = schemetools.SchemeNumber(4)

    r'''
    \new Staff \with {
        \override StaffSymbol #'thickness = #4
    } {
    }
    '''

    assert t.format == "\\new Staff \\with {\n\t\\override StaffSymbol #'thickness = #4\n} {\n}"



#from abjad.tools.componenttools._Component import _Component
#from abjad.tools.contexttools.ContextMark import ContextMark
from abjad.tools.marktools.Mark import Mark


# TODO: Make Markup.direction_string public and read / write.

#class Markup(ContextMark):
class Markup(Mark):
    r'''Abjad model of backslash-style LilyPond markup or Scheme-style LilyPond markup.

    Initialize backslash-style markup from string::

        abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }')

    ::

        abjad> markup
        Markup('\\bold { "This is markup text." }')

    ::

        abjad> f(markup)
        \markup { \bold { "This is markup text." } }

    Initialize Scheme-style markup from string::

        abjad> markup = markuptools.Markup("(markup #:draw-line '(0 . -1))", style_string = 'scheme')

    ::

        abjad> markup
        Markup("(markup #:draw-line '(0 . -1))")

    ::

        abjad> f(markup)
        #(markup #:draw-line '(0 . -1))

    Initialize any markup from existing markup::

        abjad> markup_1 = markuptools.Markup('foo', direction_string = 'up')
        abjad> markup_2 = markuptools.Markup(markup_1, direction_string = 'down')

    ::

        abjad> f(markup_1)
        ^ \markup { foo }

    ::

        abjad> f(markup_2)
        _ \markup { foo }

    Attach markup to score components like this::

        abjad> note = Note("c'4")

    ::

        abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }')

    ::

        abjad> markup(note)
        Markup('\\bold { "This is markup text." }')

    ::

        abjad> f(note)
        c'4 \markup { \bold { "This is markup text." } }

    Set `direction_string` to ``'up'``, ``'down'``, ``'neutral'`` or none.

    Set `style_string` to ``'backslash'`` or ``'scheme'``.

    Markup objects are immutable.
    '''

    __slots__ = ('_contents_string', '_direction_string', '_format_slot', '_style_string')

    def __init__(self, arg, direction_string = None, style_string = 'backslash'):
        #ContextMark.__init__(self, target_context = _Component)
        Mark.__init__(self)
        if isinstance(arg, str):
            contents_string = arg
            style_string = style_string
        elif isinstance(arg, Markup):
            contents_string = arg._contents_string
            style_string = arg._style_string
        else:
            contents_string = str(arg)
        self._contents_string = contents_string
        self._direction_string = direction_string
        self._style_string = style_string
        self._format_slot = 'right'

    ### PRIVATE ATTRIBUTES ###

    _direction_string_to_direction_symbol = {'up': '^', 'down': '_', 'neutral': '-', None: '-'}

    _style_strings = ('backslash', 'scheme')

    ### OVERLOADS ###

    def __copy__(self, *args):
        return type(self)(self._contents_string,
            direction_string = self._direction_string, style_string = self._style_string)

    __deepcopy__ = __copy__

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.format == arg.format:
                return True
        return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        if self._direction_string is not None:
            return '%s(%r, %r)' % (type(self).__name__, 
                self._contents_string, self._direction_string)
        else:
            return '%s(%r)' % (type(self).__name__, self._contents_string)

    def __str__(self):
        return self.format

    ### PUBLIC ATTRIBUTES ###

#   @property
#   def contents_string(self):
#      r'''Read-only contents string of markup:
#
#      ::
#
#         abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }')
#         abjad> markup.contents_string
#         '\\bold { "This is markup text." }'
#      '''
#      return self._contents_string
#
#   @property
#   def direction_string(self):
#      r'''Read-only direction string of markup:
#
#      ::
#
#         abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }', direction_string = 'up')
#         abjad> markup.direction_string
#         'up'
#      '''
#      return self._direction_string

    @property
    def format(self):
        r'''Read-only LilyPond format of markup::

            abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }')
            abjad> markup.format
            '\\markup { \\bold { "This is markup text." } }'

        Return string.
        '''
        result = ''
        if self._style_string == 'backslash':
            result = r'\markup { %s }' % self._contents_string
        elif self._style_string == 'scheme':
            result = '#%s' % self._contents_string
        else:
            raise ValueError('unknown markup style string: "%s".' % self._style_string)
        direction_string = self._direction_string
        if direction_string is not None:
            direction_symbol = self._direction_string_to_direction_symbol[direction_string]
            result = '%s %s' % (direction_symbol, result)
        return result

#   @property
#   def style_string(self):
#      r'''Read-only style string of markup:
#
#      ::
#
#         abjad> markup = markuptools.Markup(r'\bold { "This is markup text." }')
#         abjad> markup.style_string
#         'backslash'
#      '''
#      return self._style_string



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools._KeyboardInstrument import _KeyboardInstrument


class Piano(_KeyboardInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the piano::

        abjad> piano_staff = scoretools.PianoStaff([Staff("c'8 d'8 e'8 f'8"), Staff("c'4 b4")])

    ::

        abjad> instrumenttools.Piano()(piano_staff)
        Piano('Piano', 'Pf.')(PianoStaff<<2>>)

    ::

        abjad> f(piano_staff)
        \new PianoStaff <<
            \set PianoStaff.instrumentName = \markup { Piano }
            \set PianoStaff.shortInstrumentName = \markup { Pf. }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                c'4
                b4
            }
        >>

    The piano target piano staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Piano', short_instrument_name = 'Pf.', target_context = None):
        if target_context is None:
            target_context = scoretools.PianoStaff
        _KeyboardInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (-39, 48)



from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_partition_sequence_once_by_weights_at_most_without_overhang_01():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_once_by_weights_at_most_without_overhang(
        sequence, [10, 4])
    assert groups == [[3, 3, 3], [3]]



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Clarinet import Clarinet


class BassClarinet(Clarinet):
    r'''.. versionadded:: 2.0

    Abjad model of the bass clarinet::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.BassClarinet()(staff)
        BassClarinet('Bass Clarinet', 'Bass Cl.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Bass Clarinet }
            \set Staff.shortInstrumentName = \markup { Bass Cl. }
            c'8
            d'8
            e'8
            f'8
        }

    The bass clarinet targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Bass Clarinet', short_instrument_name = 'Bass Cl.', target_context = None):
        Clarinet.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('bf,')
        self.primary_clefs = [contexttools.ClefMark('treble')]
        self.all_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self.traditional_range = (-26, 19)



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction
import py.test


def test_intervaltreetools_scale_interval_offsets_by_rational_01():
    a = BoundedInterval(0, 10, {'a': 1})
    b = BoundedInterval(Fraction(5, 3), 8, {'b': 2})
    c = BoundedInterval(5, Fraction(61, 7), {'c': 3})
    tree = IntervalTree([a, b, c])
    scalar = Fraction(5, 2)
    scaled = scale_interval_offsets_by_rational(tree, scalar)
    assert scaled[0] == BoundedInterval(0, 10, {'a': 1})
    assert scaled[1] == BoundedInterval(Fraction(25, 6), Fraction(21, 2), {'b': 2})
    assert scaled[2] == BoundedInterval(Fraction(25, 2), Fraction(227, 14), {'c': 3})
    assert scaled[0].magnitude == a.magnitude
    assert scaled[1].magnitude == b.magnitude
    assert scaled[2].magnitude == c.magnitude

def test_intervaltreetools_scale_interval_offsets_by_rational_02():
    tree = IntervalTree([])
    scalar = Fraction(5, 2)
    scaled = scale_interval_offsets_by_rational(tree, scalar)
    assert scaled == tree



from abjad.tools.pitchtools.calculate_melodic_diatonic_interval_from_named_chromatic_pitch_to_named_chromatic_pitch import calculate_melodic_diatonic_interval_from_named_chromatic_pitch_to_named_chromatic_pitch


def calculate_harmonic_diatonic_interval_class_from_named_chromatic_pitch_to_named_chromatic_pitch(
    pitch_carrier_1, pitch_carrier_2):
    '''.. versionadded:: 2.0

    Calculate harmonic diatonic interval-class from `pitch_carrier_1` to
    `pitch_carrier_2`::

        abjad> pitchtools.calculate_harmonic_diatonic_interval_class_from_named_chromatic_pitch_to_named_chromatic_pitch(pitchtools.NamedChromaticPitch(-2), pitchtools.NamedChromaticPitch(12))
        HarmonicDiatonicIntervalClass('M2')

    Return harmonic diatonic interval-class.
    '''

    # get melodic diatonic interval
    mdi = calculate_melodic_diatonic_interval_from_named_chromatic_pitch_to_named_chromatic_pitch(
        pitch_carrier_1, pitch_carrier_2)

    # return harmonic diatonic interval-class
    return mdi.harmonic_diatonic_interval.harmonic_diatonic_interval_class




from abjad.tools import contexttools
from abjad.tools import pitchtools


class _Instrument(contexttools.InstrumentMark):
    '''.. versionadded:: 2.0

    Abjad model of the musical instrument.
    '''

    def __init__(self, instrument_name, short_instrument_name, target_context):
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch("c'")
        contexttools.InstrumentMark.__init__(
            self, instrument_name, short_instrument_name, target_context)

    ### PRIVATE METHODS ###

    def _copy_primary_clefs_to_all_clefs(self):
        self.all_clefs = [contexttools.ClefMark(clef) for clef in self.primary_clefs]

    ### PUBLIC ATTRIBUTES ###

    @apply
    def all_clefs():
        def fset(self, clefs):
            r'''Read / write all clefs.

            Return tuple of clefs.
            '''
            clefs = [contexttools.ClefMark(clef) for clef in clefs]
            self._all_clefs = clefs
        def fget(self):
            return self._all_clefs
        return property(**locals())

    @property
    def interval_of_transposition(self):
        r'''Read-only interval of transposition.

        Return melodic diatonic interval.
        '''
        return pitchtools.NamedChromaticPitch("c'") - self.sounding_pitch_of_fingered_middle_c


    @property
    def is_transposing(self):
        r'''True when instrument is transposing. False otherwise.

        Return boolean.
        '''
        return not self.sounding_pitch_of_fingered_middle_c == pitchtools.NamedChromaticPitch("c'")

    @apply
    def primary_clefs():
        def fset(self, clefs):
            r'''Read / write primary clefs.

            Return tuple of clefs.
            '''
            clefs = [contexttools.ClefMark(clef) for clef in clefs]
            self._primary_clefs = clefs
        def fget(self):
            return self._primary_clefs
        return property(**locals())

    @apply
    def sounding_pitch_of_fingered_middle_c():
        def fset(self, pitch):
            r'''Read / write sounding pitch of fingered middle C.

            Return named chromatic pitch.
            '''
            pitch = pitchtools.NamedChromaticPitch(pitch)
            self._sounding_pitch_of_fingered_middle_c = pitch
        def fget(self):
            return self._sounding_pitch_of_fingered_middle_c
        return property(**locals())

    @apply
    def traditional_range():
        def fset(self, range):
            r'''Read / write traditional range.

            Return pitch range.
            '''
            range = pitchtools.PitchRange(range)
            self._traditional_range = range
        def fget(self):
            return self._traditional_range
        return property(**locals())



from abjad.tools.intervaltreetools import *
import py.test


def test_BoundedInterval_is_contained_by_interval_01():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-7, -3)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_02():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-6, -2)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_03():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-4, 0)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_04():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-2, 0)
    assert not a.is_contained_by_interval(b)
    assert b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_05():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-2, 2)
    assert a.is_contained_by_interval(b)
    assert b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_06():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(-1, 1)
    assert not a.is_contained_by_interval(b)
    assert b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_07():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(0, 2)
    assert not a.is_contained_by_interval(b)
    assert b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_08():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(0, 4)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_09():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(2, 6)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)

def test_BoundedInterval_is_contained_by_interval_10():
    a = BoundedInterval(-2, 2)
    b = BoundedInterval(3, 7)
    assert not a.is_contained_by_interval(b)
    assert not b.is_contained_by_interval(a)



from abjad import *
import py.test


def test_SchemeFunction___setattr___01():
    '''Scheme functions are immutable.
    '''

    scheme_function = schemetools.SchemeFunction('magstep', -3)
    assert py.test.raises(AttributeError, "scheme_function.foo = 'bar'")



from abjad.tools import durationtools


def is_bar_line_crossing_leaf(leaf):
    r'''.. versionadded:: 2.0

    True when `leaf` crosses bar line::

        abjad> t = Staff("c'8 d'8 e'8 f'8")
        abjad> t[2].written_duration *= 2
        abjad> contexttools.TimeSignatureMark((2, 8), partial = Duration(1, 8))(t[2])
        TimeSignatureMark((2, 8), partial = Duration(1, 8))(e'4)
        abjad> f(t)
        \new Staff {
            c'8
            d'8
            \partial 8
            \time 2/8
            e'4
            f'8
        }
        abjad> leaftools.is_bar_line_crossing_leaf(t.leaves[2])
        True

    Otherwise false::

        abjad> leaftools.is_bar_line_crossing_leaf(t.leaves[3])
        False

    Return boolean.
    '''
    from abjad.tools import contexttools

    meter = contexttools.get_effective_time_signature(leaf)
    partial = meter.partial
    if meter.partial is None:
        partial = Duration(0)

    shifted_start = (leaf._offset.start - partial) % meter.duration
    shifted_stop = (leaf._offset.stop - partial) % meter.duration

    if meter.duration < shifted_start + leaf.prolated_duration:
        return True

    return False


